"""Flow component for managing the flow of data between nodes."""

from typing import Any, Literal, Mapping, Sequence, TypedDict

import reflex
from reflex.event import (
    JavascriptPointerEvent,
    PointerEventInfo,
    passthrough_event_spec,
    pointer_event_spec,
)

from .types import (
    AriaLabelConfig,
    ColorMode,
    Connection,
    ConnectionInProgress,
    CoordinateExtent,
    DefaultEdgeOptions,
    Edge,
    EdgeChange,
    FitViewOptions,
    HandleType,
    NoConnection,
    Node,
    NodeChange,
    NodeOrigin,
    OnConnectStartParams,
    OnDelete,
    OnDeleteParams,
    PanelPosition,
    Position,
    ProOptions,
    SnapGrid,
    Viewport,
    XYPosition,
)
from .types import EdgeAddChange as EdgeAddChange
from .types import EdgeRemoveChange as EdgeRemoveChange
from .types import EdgeReplaceChange as EdgeReplaceChange
from .types import EdgeSelectionChange as EdgeSelectionChange
from .types import NodeAddChange as NodeAddChange
from .types import NodeDimensionChange as NodeDimensionChange
from .types import NodePositionChange as NodePositionChange
from .types import NodeRemoveChange as NodeRemoveChange
from .types import NodeReplaceChange as NodeReplaceChange
from .types import NodeSelectionChange as NodeSelectionChange

LIBRARY = "@xyflow/react@12.8.4"


def _on_delete_spec(event: reflex.Var[OnDelete]) -> tuple[reflex.Var[OnDeleteParams]]:
    return (event.to(dict).params,)


def _flip_mouse_event_and_node_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent], node: reflex.Var[Node]
) -> tuple[reflex.Var[Node], reflex.Var[PointerEventInfo]]:
    return (node, pointer_event_spec(event)[0])


def _flip_mouse_event_and_edge_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent], edge: reflex.Var[Edge]
) -> tuple[reflex.Var[Edge], reflex.Var[PointerEventInfo]]:
    return (edge, pointer_event_spec(event)[0])


def _on_reconnect_start_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent],
    edge: reflex.Var[Edge],
    handle_type: reflex.Var[HandleType],
) -> tuple[reflex.Var[Edge], reflex.Var[HandleType], reflex.Var[PointerEventInfo]]:
    return (edge, handle_type, pointer_event_spec(event)[0])


def _on_reconnect_end_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent],
    edge: reflex.Var[Edge],
    handle_type: reflex.Var[HandleType],
    connection_state: reflex.Var[NoConnection | ConnectionInProgress],
) -> tuple[
    reflex.Var[Edge],
    reflex.Var[HandleType],
    reflex.Var[NoConnection | ConnectionInProgress],
    reflex.Var[PointerEventInfo],
]:
    return (edge, handle_type, connection_state, pointer_event_spec(event)[0])


def _on_connect_start_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent],
    params: reflex.Var[OnConnectStartParams],
) -> tuple[reflex.Var[OnConnectStartParams], reflex.Var[PointerEventInfo]]:
    return (params, pointer_event_spec(event)[0])


def _on_connect_end_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent],
    connection_state: reflex.Var[NoConnection | ConnectionInProgress],
) -> tuple[
    reflex.Var[NoConnection | ConnectionInProgress], reflex.Var[PointerEventInfo]
]:
    return (connection_state, pointer_event_spec(event)[0])


def _flip_mouse_event_and_viewport_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent | None],
    viewport: reflex.Var[Viewport],
) -> tuple[reflex.Var[Viewport], reflex.Var[PointerEventInfo | None]]:
    return (viewport, reflex.cond(event.is_none(), None, pointer_event_spec(event)[0]))  # pyright: ignore[reportArgumentType]


class Flow(reflex.Component):
    """The <ReactFlow /> component is the heart of your React Flow application. It renders your nodes and edges, handles user interaction, and can manage its own state if used as an uncontrolled flow."""

    library = LIBRARY

    tag = "ReactFlow"

    is_default = False

    width: reflex.Var[float]
    """Sets a fixed width for the flow."""

    height: reflex.Var[float]
    """Sets a fixed height for the flow."""

    nodes: reflex.Var[Sequence[Node]]
    """An array of nodes to render in a controlled flow."""

    edges: reflex.Var[Sequence[Edge]]
    """An array of edges to render in a controlled flow."""

    default_nodes: reflex.Var[Sequence[Node]]
    """The initial nodes to render in an uncontrolled flow."""

    default_edges: reflex.Var[Sequence[Edge]]
    """The initial edges to render in an uncontrolled flow."""

    pane_click_distance: reflex.Var[float]
    """Distance that the mouse can move between mousedown/up that will trigger a click."""

    node_click_distance: reflex.Var[float]
    """Distance that the mouse can move between mousedown/up that will trigger a click."""

    node_types: reflex.Var[Mapping[str, Any]]
    """Custom node types to be available in a flow. React Flow matches a node’s type to a component in the nodeTypes object."""

    edge_types: reflex.Var[Mapping[str, Any]]
    """Custom edge types to be available in a flow. React Flow matches an edge’s type to a component in the edgeTypes object."""

    auto_pan_on_node_focus: reflex.Var[bool]
    """When true, the viewport will pan when a node is focused."""

    node_origin: reflex.Var[NodeOrigin]
    """The origin of the node to use when placing it in the flow or looking up its x and y position. An origin of [0, 0] means that a node’s top left corner will be placed at the x and y position."""

    pro_options: reflex.Var[ProOptions]
    """By default, we render a small attribution in the corner of your flows that links back to the project.

    Anyone is free to remove this attribution whether they’re a Pro subscriber or not but we ask that you take a quick look at our https://reactflow.dev/learn/troubleshooting/remove-attribution
    removing attribution guide before doing so.
    """

    node_drag_threshold: reflex.Var[float]
    """With a threshold greater than zero you can delay node drag events. If threshold equals 1, you need to drag the node 1 pixel before a drag event is fired. 1 is the default value, so that clicks don’t trigger drag events."""

    connection_drag_threshold: reflex.Var[float]
    """The threshold in pixels that the mouse must move before a connection line starts to drag. This is useful to prevent accidental connections when clicking on a handle."""

    color_mode: reflex.Var[ColorMode]
    """Controls color scheme used for styling the flow."""

    debug: reflex.Var[bool]
    """If set true, some debug information will be logged to the console like which events are fired."""

    aria_label_config: reflex.Var[AriaLabelConfig]
    """Configuration for customizable labels, descriptions, and UI text.

    Provided keys will override the corresponding defaults.
    Allows localization, customization of ARIA descriptions, control labels, minimap labels, and other UI strings.
    """

    default_viewport: reflex.Var[Viewport]
    """Sets the initial position and zoom of the viewport. If a default viewport is provided but fitView is enabled, the default viewport will be ignored."""

    viewport: reflex.Var[Viewport]
    """When you pass a viewport prop, it’s controlled, and you also need to pass onViewportChange to handle internal changes."""

    on_viewport_change: reflex.EventHandler[passthrough_event_spec(Viewport)]
    """Used when working with a controlled viewport for updating the user viewport state."""

    fit_view: reflex.Var[bool]
    """When true, the flow will be zoomed and panned to fit all the nodes initially provided."""

    fit_view_options: reflex.Var[FitViewOptions]
    """When you typically call fitView on a ReactFlowInstance, you can provide an object of options to customize its behavior.

    This prop lets you do the same for the initial fitView call.
    """

    min_zoom: reflex.Var[float]
    """Minimum zoom level."""

    max_zoom: reflex.Var[float]
    """Maximum zoom level."""

    snap_to_grid: reflex.Var[bool]
    """When enabled, nodes will snap to the grid when dragged."""

    snap_grid: reflex.Var[SnapGrid]
    """If snapToGrid is enabled, this prop configures the grid that nodes will snap to."""

    only_render_visible_elements: reflex.Var[bool]
    """You can enable this optimisation to instruct React Flow to only render nodes and edges that would be visible in the viewport.

    This might improve performance when you have a large number of nodes and edges but also adds an overhead.
    """

    translate_extent: reflex.Var[CoordinateExtent]
    """By default, the viewport extends infinitely. You can use this prop to set a boundary.

    The first pair of coordinates is the top left boundary and the second pair is the bottom right.
    """

    node_extent: reflex.Var[CoordinateExtent]
    """By default, nodes can be placed on an infinite flow. You can use this prop to set a boundary.

    The first pair of coordinates is the top left boundary and the second pair is the bottom right.
    """

    prevent_scrolling: reflex.Var[bool]
    """Disabling this prop will allow the user to scroll the page even when their pointer is over the flow."""

    attribution_position: reflex.Var[PanelPosition]
    """By default, React Flow will render a small attribution in the bottom right corner of the flow.

    You can use this prop to change its position in case you want to place something else there.
    """

    elevate_edges_on_select: reflex.Var[bool]
    """Enabling this option will raise the z-index of edges when they are selected."""

    default_marker_color: reflex.Var[str | None]
    """Color of edge markers. You can pass null to use the CSS variable --xy-edge-stroke for the marker color."""

    default_edge_options: reflex.Var[DefaultEdgeOptions]
    """Defaults to be applied to all new edges that are added to the flow. Properties on a new edge will override these defaults if they exist."""

    reconnect_radius: reflex.Var[float]
    """The radius around an edge connection that can trigger an edge reconnection."""

    edges_reconnectable: reflex.Var[bool]
    """Whether edges can be updated once they are created. When both this prop is true and an onReconnect handler is provided, the user can drag an existing edge to a new source or target.

    Individual edges can override this value with their reconnectable property.
    """

    on_error: reflex.EventHandler[passthrough_event_spec(str, str)]
    """Occasionally something may happen that causes React Flow to throw an error.

    Instead of exploding your application, we log a message to the console and then call this event handler. You might use it for additional logging or to show a message to the user.
    """

    on_delete: reflex.EventHandler[_on_delete_spec]
    """This event handler gets called when a node or edge is deleted."""

    # Ignored because callbacks are weird
    # on_before_delete

    on_node_click: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when a user clicks on a node."""

    on_node_double_click: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when a user double-clicks on a node."""

    on_node_drag_starts: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when a user starts to drag a node."""

    on_node_drag: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when a user drags a node."""

    on_node_drag_stop: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when a user stops dragging a node."""

    on_node_mouse_enter: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when mouse of a user enters a node."""

    on_node_mouse_move: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when mouse of a user moves over a node."""

    on_node_mouse_leave: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when mouse of a user leaves a node."""

    on_node_context_menu: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """This event handler is called when a user right-clicks on a node."""

    on_nodes_delete: reflex.EventHandler[passthrough_event_spec(list[Node])]
    """This event handler gets called when a node is deleted."""

    on_nodes_change: reflex.EventHandler[passthrough_event_spec(list[NodeChange])]
    """Use this event handler to add interactivity to a controlled flow. It is called on node drag, select, and move."""

    on_edge_click: reflex.EventHandler[_flip_mouse_event_and_edge_spec]
    """This event handler is called when a user clicks on an edge."""

    on_edge_double_click: reflex.EventHandler[_flip_mouse_event_and_edge_spec]
    """This event handler is called when a user double-clicks on an edge."""

    on_edge_mouse_enter: reflex.EventHandler[_flip_mouse_event_and_edge_spec]
    """This event handler is called when mouse of a user enters an edge."""

    on_edge_mouse_move: reflex.EventHandler[_flip_mouse_event_and_edge_spec]
    """This event handler is called when mouse of a user moves over an edge."""

    on_edge_mouse_leave: reflex.EventHandler[_flip_mouse_event_and_edge_spec]
    """This event handler is called when mouse of a user leaves an edge."""

    on_edge_context_menu: reflex.EventHandler[_flip_mouse_event_and_edge_spec]
    """This event handler is called when a user right-clicks on an edge."""

    on_reconnect: reflex.EventHandler[passthrough_event_spec(Edge, Connection)]
    """This handler is called when the source or target of a reconnectable edge is dragged from the current node.

    It will fire even if the edge’s source or target do not end up changing. You can use the reconnectEdge utility to convert the connection to a new edge.
    """

    on_reconnect_start: reflex.EventHandler[_on_reconnect_start_spec]
    """This event fires when the user begins dragging the source or target of an editable edge."""

    on_reconnect_end: reflex.EventHandler[_on_reconnect_end_spec]
    """This event fires when the user releases the source or target of an editable edge. It is called even if an edge update does not occur."""

    on_edges_delete: reflex.EventHandler[passthrough_event_spec(list[Edge])]
    """This event handler gets called when an edge is deleted."""

    on_edges_change: reflex.EventHandler[passthrough_event_spec(list[EdgeChange])]
    """Use this event handler to add interactivity to a controlled flow. It is called on edge select and remove."""

    on_connect: reflex.EventHandler[passthrough_event_spec(Connection)]
    """When a connection line is completed and two nodes are connected by the user, this event fires with the new connection.

    You can use the addEdge utility to convert the connection to a complete edge.
    """

    on_connect_start: reflex.EventHandler[_on_connect_start_spec]
    """This event handler gets called when a user starts to drag a connection line."""

    on_connect_end: reflex.EventHandler[_on_connect_end_spec]
    """This callback will fire regardless of whether a valid connection could be made or not. You can use the second connectionState parameter to have different behavior when a connection was unsuccessful."""

    on_click_connect_start: reflex.EventHandler[_on_connect_start_spec]
    on_click_connect_end: reflex.EventHandler[_on_connect_end_spec]

    # Ignored as callables are weird
    # is_valid_connection: Callable[[Edge | Connection], bool]  # noqa: ERA001

    on_move: reflex.EventHandler[_flip_mouse_event_and_viewport_spec]
    """This event handler is called while the user is either panning or zooming the viewport."""

    on_move_start: reflex.EventHandler[_flip_mouse_event_and_viewport_spec]
    """This event handler is called when the user begins to pan or zoom the viewport."""

    on_move_end: reflex.EventHandler[_flip_mouse_event_and_viewport_spec]
    """This event handler is called when panning or zooming viewport movement stops. If the movement is not user-initiated, the event parameter will be null."""

    on_pane_click: reflex.EventHandler[reflex.event.pointer_event_spec]
    """This event handler gets called when user clicks inside the pane."""

    on_pane_context_menu: reflex.EventHandler[reflex.event.pointer_event_spec]
    """This event handler gets called when user right clicks inside the pane."""

    # TODO: Pass WheelEvent
    on_pane_scroll: reflex.EventHandler[reflex.event.no_args_event_spec]
    """This event handler gets called when user scroll inside the pane."""

    on_pane_mouse_move: reflex.EventHandler[reflex.event.pointer_event_spec]
    """This event handler gets called when mouse moves over the pane."""

    on_pane_mouse_enter: reflex.EventHandler[reflex.event.pointer_event_spec]
    """This event handler gets called when mouse enters the pane."""

    on_pane_mouse_leave: reflex.EventHandler[reflex.event.pointer_event_spec]
    """This event handler gets called when mouse leaves the pane."""

    # TODO: Wrap Selection Events

    nodes_draggable: reflex.Var[bool]
    """Controls whether all nodes should be draggable or not.

    Individual nodes can override this setting by setting their draggable prop.

    If you want to use the mouse handlers on non-draggable nodes, you need to add the "nopan" class to those nodes.
    """

    nodes_connectable: reflex.Var[bool]
    """Controls whether all nodes should be connectable or not.

    Individual nodes can override this setting by setting their connectable prop.
    """

    nodes_focusable: reflex.Var[bool]
    """When true, focus between nodes can be cycled with the Tab key and selected with the Enter key.

    This option can be overridden by individual nodes by setting their focusable prop.
    """

    edges_focusable: reflex.Var[bool]
    """When true, focus between edges can be cycled with the Tab key and selected with the Enter key.

    This option can be overridden by individual edges by setting their focusable prop.
    """

    elements_selectable: reflex.Var[bool]
    """When true, elements (nodes and edges) can be selected by clicking on them. This option can be overridden by individual elements by setting their selectable prop."""

    auto_pan_on_connect: reflex.Var[bool]
    """When true, the viewport will pan automatically when the cursor moves to the edge of the viewport while creating a connection"""

    auto_pan_on_node_drag: reflex.Var[bool]
    """When true, the viewport will pan automatically when the cursor moves to the edge of the viewport while dragging a node."""

    auto_pan_speed: reflex.Var[float]
    """The speed at which the viewport pans while dragging a node or a selection box."""

    pan_on_drag: reflex.Var[bool | Sequence[float]]
    """Enabling this prop allows users to pan the viewport by clicking and dragging.

    You can also set this prop to an array of numbers to limit which mouse buttons can activate panning.
    """

    selection_on_drag: reflex.Var[bool]
    """Select multiple elements with a selection box, without pressing down selectionKey."""

    selection_mode: reflex.Var[Literal["partial", "full"]]
    """When set to "partial", when the user creates a selection box by click and dragging nodes that are only partially in the box are still selected."""

    pan_on_scroll: reflex.Var[bool]
    """Controls if the viewport should pan by scrolling inside the container. Can be limited to a specific direction with panOnScrollMode."""

    pan_on_scroll_speed: reflex.Var[float]
    """Controls how fast viewport should be panned on scroll. Use together with panOnScroll prop."""

    pan_on_scroll_mode: reflex.Var[Literal["horizontal", "vertical", "free"]]
    """This prop is used to limit the direction of panning when panOnScroll is enabled. The "free" option allows panning in any direction."""

    zoom_on_scroll: reflex.Var[bool]
    """Controls if the viewport should zoom by scrolling inside the container."""

    zoom_on_pinch: reflex.Var[bool]
    """Controls if the viewport should zoom by pinching on a touch screen."""

    zoom_on_double_click: reflex.Var[bool]
    """Controls if the viewport should zoom by double-clicking somewhere on the flow."""

    select_nodes_on_drag: reflex.Var[bool]
    """If true, nodes get selected on drag."""

    elevate_nodes_on_select: reflex.Var[bool]
    """Enabling this option will raise the z-index of nodes when they are selected."""

    connect_on_click: reflex.Var[bool]
    """The connectOnClick option lets you click or tap on a source handle to start a connection and then click on a target handle to complete the connection.

    If you set this option to false, users will need to drag the connection line to the target handle to create a connection.
    """

    connection_mode: reflex.Var[Literal["strict", "loose"]]
    """A loose connection mode will allow you to connect handles with differing types, including source-to-source connections. However, it does not support target-to-target connections.

    Strict mode allows only connections between source handles and target handles.
    """

    connection_line_style: reflex.Var[Mapping[str, Any]]
    """Styles to be applied to the connection line."""

    connection_line_type: reflex.Var[
        Literal["default", "straight", "step", "smoothstep", "simplebezier"]
    ]
    """The type of edge path to use for connection lines. Although created edges can be of any type, React Flow needs to know what type of path to render for the connection line before the edge is created!"""

    connection_radius: reflex.Var[float]
    """The radius around a handle where you drop a connection line to create a new edge."""

    connection_line_component: reflex.Var[Any]
    """React Component to be used as a connection line."""

    connection_line_container_style: reflex.Var[Mapping[str, Any]]
    """Styles to be applied to the container of the connection line."""

    delete_key_code: reflex.Var[str | Sequence[str] | None]
    """If set, pressing the key or chord will delete any selected nodes and edges. Passing an array represents multiple keys that can be pressed.

    For example, ["Delete", "Backspace"] will delete selected elements when either key is pressed."""

    selection_key_code: reflex.Var[str | Sequence[str] | None]
    """If set, holding this key will let you click and drag to draw a selection box around multiple nodes and edges. Passing an array represents multiple keys that can be pressed.

    For example, ["Shift", "Meta"] will allow you to draw a selection box when either key is pressed."""

    multi_selection_key_code: reflex.Var[str | Sequence[str] | None]
    """Pressing down this key you can select multiple elements by clicking."""

    zoom_activation_key_code: reflex.Var[str | Sequence[str] | None]
    """If a key is set, you can zoom the viewport while that key is held down even if panOnScroll is set to false.

    By setting this prop to null you can disable this functionality."""

    pan_activation_key_code: reflex.Var[str | Sequence[str] | None]
    """If a key is set, you can pan the viewport while that key is held down even if panOnScroll is set to false.

    By setting this prop to null you can disable this functionality."""

    disable_keyboard_a11y: reflex.Var[bool]
    """You can use this prop to disable keyboard accessibility features such as selecting nodes or moving selected nodes with the arrow keys."""

    no_pan_class_name: reflex.Var[str]
    """If an element in the canvas does not stop mouse events from propagating, clicking and dragging that element will pan the viewport. Adding the "nopan" class prevents this behavior and this prop allows you to change the name of that class."""

    no_drag_class_name: reflex.Var[str]
    """If a node is draggable, clicking and dragging that node will move it around the canvas. Adding the "nodrag" class prevents this behavior and this prop allows you to change the name of that class."""

    no_wheel_class_name: reflex.Var[str]
    """Typically, scrolling the mouse wheel when the mouse is over the canvas will zoom the viewport. Adding the "nowheel" class to an element in the canvas will prevent this behavior and this prop allows you to change the name of that class."""

    def add_imports(self) -> reflex.ImportDict:
        """Add the imports for the datatable component.

        Returns:
            The import dict for the component.
        """
        return {"": "@xyflow/react/dist/style.css"}


class FlowProvider(reflex.Component):
    """The <ReactFlowProvider /> component is a context provider  that makes it possible to access a flow’s internal state outside of the <ReactFlow /> component. Many of the hooks we provide rely on this component to work."""

    library = LIBRARY

    tag = "ReactFlowProvider"

    is_default = False

    initial_nodes: reflex.Var[Sequence[Node]]
    """These nodes are used to initialize the flow. They are not dynamic."""

    initial_edges: reflex.Var[Sequence[Edge]]
    """These edges are used to initialize the flow. They are not dynamic."""

    default_nodes: reflex.Var[Sequence[Node]]
    """These nodes are used to initialize the flow. They are not dynamic."""

    default_edges: reflex.Var[Sequence[Edge]]
    """These edges are used to initialize the flow. They are not dynamic."""

    initial_width: reflex.Var[float]
    """The initial width is necessary to be able to use fitView on the server."""

    initial_height: reflex.Var[float]
    """The initial height is necessary to be able to use fitView on the server."""

    fit_view: reflex.Var[bool]
    """When true, the flow will be zoomed and panned to fit all the nodes initially provided."""

    initial_fit_view_options: reflex.Var[FitViewOptions]
    """You can provide an object of options to customize the initial fitView behavior."""

    initial_min_zoom: reflex.Var[float]
    """Initial minimum zoom level."""

    initial_max_zoom: reflex.Var[float]
    """Initial maximum zoom level."""

    node_origin: reflex.Var[NodeOrigin]
    """The origin of the node to use when placing it in the flow or looking up its x and y position. An origin of [0, 0] means that a node’s top left corner will be placed at the x and y position."""

    node_extent: reflex.Var[CoordinateExtent]
    """By default, nodes can be placed on an infinite flow. You can use this prop to set a boundary.

    The first pair of coordinates is the top left boundary and the second pair is the bottom right."""


class Background(reflex.Component):
    """The <Background /> component makes it convenient to render different types of backgrounds common in node-based UIs. It comes with three variants: lines, dots and cross."""

    library = LIBRARY

    tag = "Background"

    is_default = False

    color: reflex.Var[str]
    """Color of the pattern."""

    bg_color: reflex.Var[str]
    """Color of the background."""

    pattern_class_name: reflex.Var[str]
    """Class applied to the pattern."""

    gap: reflex.Var[float | tuple[float, float]]
    """The gap between patterns. Passing in a tuple allows you to control the x and y gap independently."""

    size: reflex.Var[float]
    """The radius of each dot or the size of each rectangle if BackgroundVariant.Dots or BackgroundVariant.Cross is used. This defaults to 1 or 6 respectively, or ignored if BackgroundVariant.Lines is used."""

    offset: reflex.Var[float | tuple[float, float]]
    """Offset of the pattern."""

    line_width: reflex.Var[float]
    """The width of the line used in the pattern."""

    variant: reflex.Var[Literal["lines", "dots", "cross"]]
    """Variant of the pattern."""


class BaseEdge(reflex.Component):
    """The <BaseEdge /> component gets used internally for all the edges. It can be used inside a custom edge and handles the invisible helper edge and the edge label for you."""

    library = LIBRARY

    tag = "BaseEdge"

    is_default = False

    path: reflex.Var[str]
    """The SVG path string that defines the edge. This should look something like 'M 0 0 L 100 100' for a simple line. The utility functions like getSimpleBezierEdge can be used to generate this string for you."""

    marker_start: reflex.Var[str]
    """The id of the SVG marker to use at the start of the edge. This should be defined in a <defs> element in a separate SVG document or element. Use the format “url(#markerId)” where markerId is the id of your marker definition."""

    marker_end: reflex.Var[str]
    """The id of the SVG marker to use at the end of the edge. This should be defined in a <defs> element in a separate SVG document or element. Use the format “url(#markerId)” where markerId is the id of your marker definition."""

    label: reflex.Var[Any]
    """The label or custom element to render along the edge. This is commonly a text label or some custom controls."""

    label_style: reflex.Var[Mapping[str, Any]]
    """Custom styles to apply to the label."""

    label_show_bg: reflex.Var[bool]

    label_bg_style: reflex.Var[Mapping[str, Any]]

    label_bg_padding: reflex.Var[tuple[float, float]]

    label_bg_border_radius: reflex.Var[float]

    interaction_width: reflex.Var[float]
    """The width of the invisible area around the edge that the user can interact with. This is useful for making the edge easier to click or hover over."""

    label_x: reflex.Var[float]
    """The x position of edge label."""

    label_y: reflex.Var[float]
    """The y position of edge label."""


class ControlButton(reflex.el.elements.Button):
    """You can add buttons to the control panel by using the <ControlButton /> component and pass it as a child to the <Controls /> component."""

    library = LIBRARY

    tag = "ControlButton"

    is_default = False


class Controls(reflex.Component):
    """The <Controls /> component renders a small panel that contains convenient buttons to zoom in, zoom out, fit the view, and lock the viewport."""

    library = LIBRARY

    tag = "Controls"

    is_default = False

    show_zoom: reflex.Var[bool]
    """Whether or not to show the zoom in and zoom out buttons. These buttons will adjust the viewport zoom by a fixed amount each press."""

    show_fit_view: reflex.Var[bool]
    """Whether or not to show the fit view button. By default, this button will adjust the viewport so that all nodes are visible at once."""

    show_interactive: reflex.Var[bool]
    """Show button for toggling interactivity."""

    fit_view_options: reflex.Var[FitViewOptions]
    """Customise the options for the fit view button. These are the same options you would pass to the fitView function."""

    on_zoom_in: reflex.EventHandler[reflex.event.no_args_event_spec]
    """Called in addition the default zoom behavior when the zoom in button is clicked."""

    on_zoom_out: reflex.EventHandler[reflex.event.no_args_event_spec]
    """Called in addition the default zoom behavior when the zoom out button is clicked."""

    on_fit_view: reflex.EventHandler[reflex.event.no_args_event_spec]
    """Called when the fit view button is clicked. When this is not provided, the viewport will be adjusted so that all nodes are visible."""

    on_interactive_change: reflex.EventHandler[passthrough_event_spec(bool)]
    """Called when the interactive (lock) button is clicked."""

    position: reflex.Var[PanelPosition]
    """Position of the controls on the pane"""

    # TODO: wrap aria-label

    orientation: reflex.Var[Literal["horizontal", "vertical"]]


class EdgeLabelRenderer(reflex.Component):
    """Edges are SVG-based. If you want to render more complex labels you can use the <EdgeLabelRenderer /> component to access a div based renderer. This component is a portal that renders the label in a <div /> that is positioned on top of the edges. You can see an example usage of the component in the edge label renderer example."""

    library = LIBRARY

    tag = "EdgeLabelRenderer"

    is_default = False


class EdgeText(reflex.Component):
    """You can use the <EdgeText /> component as a helper component to display text within your custom edges."""

    library = LIBRARY

    tag = "EdgeText"

    is_default = False

    x: reflex.Var[float]
    """The x position where the label should be rendered."""

    y: reflex.Var[float]
    """The y position where the label should be rendered."""

    label: reflex.Var[Any]
    """The label or custom element to render along the edge. This is commonly a text label or some custom controls."""

    label_style: reflex.Var[Mapping[str, Any]]
    """Custom styles to apply to the label."""

    label_show_bg: reflex.Var[bool]

    label_bg_style: reflex.Var[Mapping[str, Any]]

    label_bg_padding: reflex.Var[tuple[float, float]]

    label_bg_border_radius: reflex.Var[float]


class Handle(reflex.Component):
    """The <Handle /> component is used in your custom nodes to define connection points."""

    library = LIBRARY

    tag = "Handle"

    is_default = False

    type: reflex.Var[Literal["source", "target"]]
    """Type of the handle."""

    position: reflex.Var[Position]
    """The position of the handle relative to the node. In a horizontal flow source handles are typically Position.Right and in a vertical flow they are typically Position.Top."""

    is_connectable: reflex.Var[bool]
    """Should you be able to connect to/from this handle."""

    is_connectable_start: reflex.Var[bool]
    """Dictates whether a connection can start from this handle."""

    is_connectable_end: reflex.Var[bool]
    """Dictates whether a connection can end on this handle."""

    # TODO: skipped is_valid_connection because callbacks are weird
    # is_valid_connection: Callable[[Edge | Connection], bool]  # noqa: ERA001
    # """Called when a connection is dragged to this handle. You can use this callback to perform some custom validation logic based on the connection target and source, for example. Where possible, we recommend you move this logic to the isValidConnection prop on the main ReactFlow component for performance reasons."""

    on_connect: reflex.EventHandler[passthrough_event_spec(Connection)]
    """Callback called when connection is made"""


def _flip_mouse_event_and_xy_position_spec(
    event: reflex.vars.ObjectVar[JavascriptPointerEvent],
    position: reflex.Var[XYPosition],
) -> tuple[reflex.Var[XYPosition], reflex.Var[PointerEventInfo]]:
    return (position, pointer_event_spec(event)[0])


class MiniMap(reflex.Component):
    """The <MiniMap /> component can be used to render an overview of your flow. It renders each node as an SVG element and visualizes where the current viewport is in relation to the rest of the flow."""

    library = LIBRARY

    tag = "MiniMap"

    is_default = False

    position: reflex.Var[PanelPosition]
    """Position of minimap on pane."""

    on_click: reflex.EventHandler[_flip_mouse_event_and_xy_position_spec]
    """Callback called when minimap is clicked."""

    node_color: reflex.Var[str | Any]
    """Color of nodes on minimap."""

    node_stroke_color: reflex.Var[str | Any]
    """Stroke color of nodes on minimap."""

    node_class_name: reflex.Var[str | Any]
    """Class name applied to nodes on minimap."""

    node_border_radius: reflex.Var[float]
    """Border radius of nodes on minimap."""

    node_stroke_width: reflex.Var[float]
    """Stroke width of nodes on minimap."""

    node_component: reflex.Var[Any]
    """A custom component to render the nodes in the minimap. This component must render an SVG element!"""

    bg_color: reflex.Var[str]
    """Background color of minimap."""

    mask_color: reflex.Var[str]
    """The color of the mask that covers the portion of the minimap not currently visible in the viewport."""

    mask_stroke_color: reflex.Var[str]
    """Stroke color of mask representing viewport."""

    mask_stroke_width: reflex.Var[float]
    """Stroke width of mask representing viewport."""

    on_node_click: reflex.EventHandler[_flip_mouse_event_and_node_spec]
    """Callback called when node on minimap is clicked."""

    pannable: reflex.Var[bool]
    """Determines whether you can pan the viewport by dragging inside the minimap."""

    zoomable: reflex.Var[bool]
    """Determines whether you can zoom the viewport by scrolling inside the minimap."""

    aria_label: str | None
    """There is no text inside the minimap for a screen reader to use as an accessible name, so it’s important we provide one to make the minimap accessible. The default is sufficient, but you may want to replace it with something more relevant to your app or product."""

    inverse_pan: reflex.Var[bool]
    """Invert direction when panning the minimap viewport."""

    zoom_step: reflex.Var[float]
    """Step size for zooming in/out on minimap."""

    offset_scale: reflex.Var[float]
    """Scale factor for offsetting the minimap viewport."""


class ResizeParams(TypedDict):
    """Parameters for resizing a node."""

    x: float
    y: float
    width: float
    height: float


class ResizeParamsWithDirection(TypedDict):
    """Parameters for resizing a node with direction."""

    direction: list[float]


ControlLinePosition = Literal["left", "right", "top", "bottom"]

ControlPosition = (
    ControlLinePosition
    | Literal["top-left", "top-right", "bottom-left", "bottom-right"]
)

ResizeControlVariant = Literal["line", "handle"]

ResizeControlDirection = Literal["horizontal", "vertical"]


class NodeResizeControl(reflex.Component):
    """To create your own resizing UI, you can use the NodeResizeControl component where you can pass children (such as icons)."""

    library = LIBRARY

    tag = "NodeResizeControl"

    is_default = False

    node_id: reflex.Var[str]
    """Id of the node it is resizing."""

    color: reflex.Var[str]
    """Color of the resize handle."""

    min_width: reflex.Var[float]
    """Minimum width of node."""

    min_height: reflex.Var[float]
    """Minimum height of node."""

    max_width: reflex.Var[float]
    """Maximum width of node."""

    max_height: reflex.Var[float]
    """Maximum height of node."""

    keep_aspect_ratio: reflex.Var[bool]
    """Keep aspect ratio when resizing."""

    # TODO: Callbacks are weird
    # should_resize: ShouldResize  # noqa: ERA001
    # """Callback to determine if node should resize."""

    auto_scale: reflex.Var[bool]
    """Scale the controls with the zoom level."""

    on_resize_start: reflex.EventHandler[passthrough_event_spec(ResizeParams)]
    """Callback called when resizing starts."""

    on_resize: reflex.EventHandler[passthrough_event_spec(ResizeParamsWithDirection)]
    """Callback called when resizing."""

    on_resize_end: reflex.EventHandler[passthrough_event_spec(ResizeParams)]
    """Callback called when resizing ends."""

    position: reflex.Var[ControlPosition]
    """Position of the control."""

    variant: reflex.Var[ResizeControlVariant]
    """Variant of the control."""

    resize_direction: reflex.Var[ResizeControlDirection]
    """The direction the user can resize the node. If not provided, the user can resize in any direction."""


class NodeResizer(reflex.Component):
    """The <NodeResizer /> component can be used to add a resize functionality to your nodes. It renders draggable controls around the node to resize in all directions."""

    library = LIBRARY

    tag = "NodeResizer"

    is_default = False

    node_id: reflex.Var[str]
    """Id of the node it is resizing."""

    color: reflex.Var[str]
    """Color of the resize handle."""

    handle_class_name: reflex.Var[str]
    """Class name applied to handle."""

    handle_style: reflex.Var[Mapping[str, Any]]
    """Style applied to handle."""

    line_class_name: reflex.Var[str]
    """Class name applied to line."""

    line_style: reflex.Var[Mapping[str, Any]]
    """Style applied to line."""

    is_visible: reflex.Var[bool]
    """Are the controls visible."""

    min_width: reflex.Var[float]
    """Minimum width of node."""

    min_height: reflex.Var[float]
    """Minimum height of node."""

    max_width: reflex.Var[float]
    """Maximum width of node."""

    max_height: reflex.Var[float]
    """Maximum height of node."""

    keep_aspect_ratio: reflex.Var[bool]
    """Keep aspect ratio when resizing."""

    auto_scale: reflex.Var[bool]
    """Scale the controls with the zoom level."""

    # TODO: Wrapping callbacks is weird
    # should_resize: ShouldResize  # noqa: ERA001
    # """Callback to determine if node should resize."""

    on_resize_start: reflex.EventHandler[passthrough_event_spec(ResizeParams)]
    """Callback called when resizing starts."""

    on_resize: reflex.EventHandler[passthrough_event_spec(ResizeParamsWithDirection)]
    """Callback called when resizing."""

    on_resize_end: reflex.EventHandler[passthrough_event_spec(ResizeParams)]
    """Callback called when resizing ends."""


Align = Literal["center", "start", "end"]


class NodeToolbar(reflex.Component):
    """This component can render a toolbar or tooltip to one side of a custom node. This toolbar doesn’t scale with the viewport so that the content is always visible."""

    library = LIBRARY

    tag = "NodeToolbar"

    is_default = False

    node_id: reflex.Var[str | Sequence[str]]
    """By passing in an array of node id’s you can render a single tooltip for a group or collection of nodes."""

    is_visible: reflex.Var[bool]
    """If true, node toolbar is visible even if node is not selected."""

    position: reflex.Var[Position]
    """Position of the toolbar relative to the node."""

    offset: reflex.Var[float]
    """The space between the node and the toolbar, measured in pixels."""

    align: reflex.Var[Align]
    """Align the toolbar relative to the node."""


class Panel(reflex.Component):
    """The <Panel /> component helps you position content above the viewport. It is used internally by the <MiniMap /> and <Controls /> components."""

    library = LIBRARY

    tag = "Panel"

    is_default = False

    position: reflex.Var[PanelPosition]
    """The position of the panel."""


class ViewportPortal(reflex.Component):
    """<ViewportPortal /> component can be used to add components to the same viewport of the flow where nodes and edges are rendered. This is useful when you want to render your own components that adhere to the same coordinate system as the nodes & edges and are also affected by zooming and panning."""

    library = LIBRARY

    tag = "ViewportPortal"

    is_default = False


flow = Flow.create
provider = FlowProvider.create
background = Background.create
base_edge = BaseEdge.create
control_button = ControlButton.create
controls = Controls.create
edge_label_renderer = EdgeLabelRenderer.create
edge_text = EdgeText.create
handle = Handle.create
mini_map = MiniMap.create
node_resize_control = NodeResizeControl.create
node_resizer = NodeResizer.create
node_toolbar = NodeToolbar.create
panel = Panel.create
viewport_portal = ViewportPortal.create



"""Stub file for reflex_enterprise/components/mantine/tree.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import Any, Mapping, Optional, Sequence, TypedDict

from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import MantineCoreBase

class TreeNodeData(TypedDict):
    value: str
    label: Any
    children: list[NestedTreeNode | LeafNode]

class NestedTreeNode(TypedDict):
    value: str
    label: Any
    children: list[NestedTreeNode2 | LeafNode]

class NestedTreeNode2(TypedDict):
    value: str
    label: Any
    children: list[NestedTreeNode3 | LeafNode]

class NestedTreeNode3(TypedDict):
    value: str
    label: Any
    children: list[NestedTreeNode4 | LeafNode]

class NestedTreeNode4(TypedDict):
    value: str
    label: Any
    children: list[LeafNode]

class LeafNode(TypedDict):
    value: str
    label: Any

class Tree(MantineCoreBase):
    @classmethod
    def create(
        cls,
        *children,
        data: Var[list[TreeNodeData]] | list[TreeNodeData] | None = None,
        label_key: Var[str] | str | None = None,
        level: Var[int] | int | None = None,
        value_key: Var[str] | str | None = None,
        value: Var[list[str]] | list[str] | None = None,
        default_value: Var[list[str]] | list[str] | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_change: Optional[EventType[Any]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_node_click: Optional[EventType[Any]] = None,
        on_node_collapse: Optional[EventType[Any]] = None,
        on_node_expand: Optional[EventType[Any]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "Tree":
        """Create the Reflex Enterprise component."""
        ...

tree = Tree.create



"""Parameter classes for AG Grid renderer configurations."""

from typing import Any, Mapping, Sequence

from reflex.vars.base import Var

from .base import JS_EXPRESSION, AgGridParamsBase, AgGridResourceBase


class DetailGridOptions(AgGridResourceBase):
    """Options for detail grid in master detail setup."""

    # Column definitions for the detail grid
    column_defs: (
        Sequence[Mapping[str, Any]] | Var[Sequence[Mapping[str, Any]]] | None
    ) = None

    # Default column definition for the detail grid
    default_col_def: Mapping[str, Any] | Var[Mapping[str, Any]] | None = None

    # Enable pagination in the detail grid
    pagination: bool | Var[bool] | None = None

    # Number of rows per page in the detail grid
    pagination_page_size: int | Var[int] | None = None

    # Enable sorting in the detail grid
    sortable: bool | Var[bool] | None = None

    # Enable filtering in the detail grid
    filterable: bool | Var[bool] | None = None

    # Row height for the detail grid
    row_height: int | Var[int] | None = None

    # Enable row selection in the detail grid
    row_selection: str | dict | Var[str] | Var[dict] | None = None


class DetailCellRendererParams(AgGridParamsBase):
    """Parameters for detail cell renderer in master detail setup."""

    # Options for the detail grid
    detail_grid_options: (
        DetailGridOptions | dict | Var[DetailGridOptions] | Var[dict] | None
    ) = None

    # Function to get data for the detail row
    get_detail_row_data: JS_EXPRESSION | None = None

    # Template for the detail row
    detail_row_template: str | Var[str] | None = None

    # Custom renderer for the detail row
    detail_cell_renderer: JS_EXPRESSION | None = None

    # Whether to refresh the detail grid when the master row data changes
    refresh_strategy: str | Var[str] | None = None


class LoadingCellRendererParams(AgGridParamsBase):
    """Parameters for loading cell renderer."""

    # Loading text to display
    loading_message: str | Var[str] | None = None

    # Custom loading renderer
    loading_renderer: JS_EXPRESSION | None = None

    # Loading icon or spinner component
    loading_icon: str | Var[str] | None = None


class FullWidthCellRendererParams(AgGridParamsBase):
    """Parameters for full width cell renderer."""

    # Custom full width renderer
    full_width_renderer: JS_EXPRESSION | None = None

    # Template for full width rows
    full_width_template: str | Var[str] | None = None

    # CSS class for full width rows
    full_width_css_class: str | Var[str] | None = None



"""Map consumer component for event handling in leaflet."""

from __future__ import annotations

import reflex as rx
from reflex.vars.base import Var

from reflex_enterprise.components.map.base import LazyBaseLeafletComponent
from reflex_enterprise.components.map.events import (
    error_event_spec,
    layer_event_spec,
    locationfound_event_spec,
    mouse_event_spec,
    move_event_spec,
    popup_event_spec,
    resize_event_spec,
    tooltip_event_spec,
    zoom_event_spec,
    zoom_levels_change_event_spec,
)

# Force this imports for the pyi file
from .types import (  # noqa: F401
    ErrorEvent,
    LatLng,
    LayerEvent,
    MouseEvent,
    MoveEvent,
    PopupEvent,
    ResizeEvent,
    TooltipEvent,
    ZoomEvent,
    ZoomLevelsChangeEvent,
)


class MapConsumer(LazyBaseLeafletComponent):
    """MapConsumer for events."""

    tag = "MapConsumer"

    map_ref: Var[str]

    # Map movement events
    on_move: rx.EventHandler[move_event_spec]
    on_move_start: rx.EventHandler[move_event_spec]
    on_move_end: rx.EventHandler[move_event_spec]

    # Mouse events
    on_click: rx.EventHandler[mouse_event_spec]
    on_dblclick: rx.EventHandler[mouse_event_spec]
    on_mousedown: rx.EventHandler[mouse_event_spec]
    on_mouseup: rx.EventHandler[mouse_event_spec]
    on_mouseover: rx.EventHandler[mouse_event_spec]
    on_mouseout: rx.EventHandler[mouse_event_spec]
    on_mousemove: rx.EventHandler[mouse_event_spec]
    on_contextmenu: rx.EventHandler[mouse_event_spec]

    # Zoom events
    on_zoom: rx.EventHandler[zoom_event_spec]
    on_zoom_start: rx.EventHandler[zoom_event_spec]
    on_zoom_end: rx.EventHandler[zoom_event_spec]
    on_zoom_levels_change: rx.EventHandler[zoom_levels_change_event_spec]

    # Location events
    on_locationfound: rx.EventHandler[locationfound_event_spec]
    on_locationerror: rx.EventHandler[error_event_spec]

    # UI events
    on_resize: rx.EventHandler[resize_event_spec]
    on_load: rx.EventHandler[rx.event.passthrough_event_spec(str)]
    on_unload: rx.EventHandler[rx.event.passthrough_event_spec(str)]
    on_viewreset: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Layer events
    on_layeradd: rx.EventHandler[layer_event_spec]
    on_layerremove: rx.EventHandler[layer_event_spec]

    # Popup/tooltip events
    on_popupopen: rx.EventHandler[popup_event_spec]
    on_popupclose: rx.EventHandler[popup_event_spec]
    on_tooltipopen: rx.EventHandler[tooltip_event_spec]
    on_tooltipclose: rx.EventHandler[tooltip_event_spec]

    # A dictionary to hold all event handlers
    events_handler: Var[dict]

    @classmethod
    def create(cls, *children, **props) -> rx.Component:
        """Create a MapConsumer.

        This method maps Python-style event handlers (on_move) to
        the Leaflet-style event names (move).
        """
        _mapping = {
            # Map state change events
            "on_move": "move",
            "on_move_start": "movestart",
            "on_move_end": "moveend",
            "on_zoom": "zoom",
            "on_zoom_start": "zoomstart",
            "on_zoom_end": "zoomend",
            "on_zoom_levels_change": "zoomlevelschange",
            "on_resize": "resize",
            "on_load": "load",
            "on_unload": "unload",
            "on_viewreset": "viewreset",
            # Pointer events
            "on_click": "click",
            "on_dblclick": "dblclick",
            "on_mousedown": "mousedown",
            "on_mouseup": "mouseup",
            "on_mouseover": "mouseover",
            "on_mouseout": "mouseout",
            "on_mousemove": "mousemove",
            "on_contextmenu": "contextmenu",
            # Location events
            "on_locationfound": "locationfound",
            "on_locationerror": "locationerror",
            # Layer events
            "on_layeradd": "layeradd",
            "on_layerremove": "layerremove",
            # Popup/tooltip events
            "on_popupopen": "popupopen",
            "on_popupclose": "popupclose",
            "on_tooltipopen": "tooltipopen",
            "on_tooltipclose": "tooltipclose",
        }

        # Call create for a temporary object that formats the events_triggers
        _consumer = super().create(*children, **props)

        props["events_handler"] = {}
        to_remove = []
        for evt in _consumer.event_triggers:
            if evt in _mapping:
                props["events_handler"][_mapping[evt]] = _consumer.event_triggers[evt]
                to_remove.append(evt)

        for rem in to_remove:
            props.pop(rem)

        return super().create(*children, **props)

    def add_hooks(self) -> list[Var]:
        """Add hooks for leaflet components."""
        hooks = []
        return hooks



"""Handle proxying frontend requests from the backend server."""

from __future__ import annotations

import asyncio
from contextlib import asynccontextmanager
from typing import Any, AsyncGenerator
from urllib.parse import urlparse

from reflex.config import get_config
from reflex.utils import console
from starlette.applications import Starlette
from starlette.routing import Mount
from starlette.types import ASGIApp, Receive, Scope, Send

from reflex_enterprise.utils import is_new_session

try:
    import aiohttp
    from asgiproxy.config import BaseURLProxyConfigMixin, ProxyConfig
    from asgiproxy.context import ProxyContext
    from asgiproxy.proxies.http import proxy_http
    from asgiproxy.simple_proxy import make_simple_proxy_app
except ImportError:
    if is_new_session():
        console.warn("asgiproxy is not installed. Proxying will be disabled.")

    @asynccontextmanager
    async def proxy_middleware(*args, **kwargs) -> AsyncGenerator[None, None]:
        """A no-op proxy middleware for when asgiproxy is not installed.

        Args:
            *args: The positional arguments.
            **kwargs: The keyword arguments.

        Yields:
            None
        """
        yield
else:
    MAX_PROXY_RETRY = 25

    async def proxy_http_with_retry(
        *,
        context: ProxyContext,
        scope: Scope,
        receive: Receive,
        send: Send,
    ) -> Any:
        """Proxy an HTTP request with retries.

        Args:
            context: The proxy context.
            scope: The request scope.
            receive: The receive channel.
            send: The send channel.

        Returns:
            The response from `proxy_http`.
        """
        for _attempt in range(MAX_PROXY_RETRY):
            try:
                return await proxy_http(
                    context=context,
                    scope=scope,
                    receive=receive,
                    send=send,
                )
            except aiohttp.ClientError as err:  # noqa: PERF203
                console.debug(
                    f"Retrying request {scope['path']} due to client error {err!r}."
                )
                await asyncio.sleep(0.3)
            except Exception as ex:
                console.debug(
                    f"Retrying request {scope['path']} due to unhandled exception {ex!r}."
                )
                await asyncio.sleep(0.3)

    def _get_proxy_app_with_context(frontend_host: str) -> tuple[ProxyContext, ASGIApp]:
        """Get the proxy app with the given frontend host.

        Args:
            frontend_host: The frontend host to proxy requests to.

        Returns:
            The proxy context and app.
        """

        class LocalProxyConfig(BaseURLProxyConfigMixin, ProxyConfig):
            upstream_base_url = frontend_host
            rewrite_host_header = urlparse(upstream_base_url).netloc

        proxy_context = ProxyContext(LocalProxyConfig())
        proxy_app = make_simple_proxy_app(
            proxy_context, proxy_http_handler=proxy_http_with_retry
        )
        return proxy_context, proxy_app

    def _get_base_starlette_app(app: ASGIApp) -> Starlette | None:
        """Get the base Starlette app from the given ASGI app.

        Args:
            app: The ASGI app.

        Returns:
            The base Starlette app.
        """
        if isinstance(app, Starlette):
            for route in app.routes:
                if (
                    isinstance(route, Mount)
                    and route.path in ["", "/"]
                    and (base_app := _get_base_starlette_app(route.app)) is not None
                ):
                    return base_app
            return app

    @asynccontextmanager
    async def proxy_middleware(  # pyright: ignore[reportRedeclaration]
        app: Starlette,
    ) -> AsyncGenerator[None, None]:
        """A middleware to proxy requests to the separate frontend server.
        The proxy is installed on the / endpoint of the Starlette instance.

        Args:
            app: The Starlette instance.

        Yields:
            None
        """
        # Find the bottom level app to mount the proxy on
        base_app = _get_base_starlette_app(app)
        if base_app is None:
            console.error(
                "Unable to find the base Starlette app. Proxying will not be enabled."
            )
            yield
            return

        config = get_config()
        backend_port = config.backend_port
        frontend_host = f"http://localhost:{config.frontend_port}"
        proxy_context, proxy_app = _get_proxy_app_with_context(frontend_host)
        base_app.mount("/", proxy_app)
        console.debug(
            f"Proxying '/' requests on port {backend_port} to {frontend_host}"
        )
        async with proxy_context:
            yield



"""Javascript functions for working with the browser."""

from reflex.vars import Var
from reflex.vars.function import ArgsFunctionOperation, FunctionStringVar

WINDOW_OPEN = FunctionStringVar.create("window.open")
CLOSE_POPUP = ArgsFunctionOperation.create(
    ("origin", "close_timeout"),
    return_expr=Var("""{
        if (window.opener && window.opener.origin === origin) {
            setTimeout(() => window.close(), close_timeout);
        }
    }"""),
)
POST_MESSAGE_AND_CLOSE_POPUP = ArgsFunctionOperation.create(
    ("payload", "origin", "close_timeout"),
    return_expr=Var("""{
        if (window.opener && window.opener.origin === origin) {
            window.opener.postMessage(payload, origin);
            setTimeout(() => window.close(), close_timeout);
        }
    }"""),
)



"""Base class for AG Grid resource objects with JS expression handling."""

import typing
from typing import Any, Callable

from reflex.components.props import PropsBase
from reflex.utils import console
from reflex.utils.format import to_camel_case, to_snake_case
from reflex.utils.types import typehint_issubclass
from reflex.vars import FunctionVar

from ..resource import value_func_factory

# JS expression type for properties that can contain JavaScript code
JS_EXPRESSION = FunctionVar | str | dict | Callable


class AgGridResourceBase(PropsBase):
    """Base class for AG Grid resource objects with JS expression handling."""

    def __init__(self, **kwargs):
        """Initialize with camelCase to snake_case conversion for kwargs.

        Args:
            **kwargs: The field values to set.
        """
        # Convert camelCase kwargs to snake_case for compatibility with JS libraries
        field_names = self.get_fields()
        converted_kwargs = {}

        for key, value in kwargs.items():
            # Try exact match first
            if key in field_names:
                converted_kwargs[key] = value
            else:
                # Try converting camelCase to snake_case
                snake_key = to_snake_case(key)
                if snake_key in field_names:
                    converted_kwargs[snake_key] = value
                else:
                    # Keep as-is for unknown fields
                    converted_kwargs[key] = value

        super().__init__(**converted_kwargs)

    @classmethod
    def get_js_expressions(cls, camel_case: bool = False) -> dict[str, Any]:
        """Get the list of fields that are JS expressions.

        Args:
            camel_case: If True, convert field names to camelCase.

        Returns:
            Dictionary mapping field names to their types for JS expression fields.
        """
        return {
            to_camel_case(key) if camel_case else key: value.type_
            for key, value in cls.get_fields().items()
            if typehint_issubclass(JS_EXPRESSION, value.type_)
        }

    def _process_js_expressions(self, return_dict: dict) -> dict:
        """Process JS expressions in the dictionary.

        Args:
            return_dict: Dictionary to process JS expressions in.

        Returns:
            Dictionary with processed JS expressions.
        """
        for key, exp_types in self.get_js_expressions(camel_case=True).items():
            js_types = (
                _js_types if (_js_types := typing.get_args(exp_types)) else (exp_types,)
            )
            if key in return_dict:
                if isinstance(return_dict[key], dict):
                    return_dict[key] = value_func_factory(
                        return_dict[key]["function"],
                        self._known_components  # pyright: ignore [reportAttributeAccessIssue]
                        if hasattr(self, "_known_components")
                        else [],
                    )
                elif isinstance(return_dict[key], str):
                    return_dict[key] = value_func_factory(
                        return_dict[key],
                        self._known_components  # pyright: ignore [reportAttributeAccessIssue]
                        if hasattr(self, "_known_components")
                        else [],
                    )
                elif not isinstance(return_dict[key], js_types):
                    console.warn(
                        f"Unexpected type for JS expression in {key}: {type(return_dict[key])}. "
                        f"Expected one of: {js_types}."
                    )
        return return_dict

    def dict(self, *args, **kwargs) -> dict[str, Any]:
        """Override dict method to exclude None values and handle JS expressions.

        Args:
            *args: Arguments passed to parent dict method.
            **kwargs: Keyword arguments passed to parent dict method.

        Returns:
            Dictionary representation with processed JS expressions.
        """
        kwargs.setdefault("exclude_none", True)
        return_dict = super().dict(*args, **kwargs)

        # Process JS expressions
        return_dict = self._process_js_expressions(return_dict)

        return return_dict


class AgGridParamsBase(AgGridResourceBase):
    """Base class for AG Grid parameter classes with auto-naming conventions."""

    @classmethod
    def get_prop_name(cls) -> str:
        """Get the snake_case prop name from the class name.

        Example: DetailCellRendererParams -> detail_cell_renderer_params

        Returns:
            The snake_case property name.
        """
        return to_snake_case(cls.__name__)

    @classmethod
    def get_formatter_function_name(cls) -> str:
        """Get the JavaScript formatter function name from the class name.

        Example: DetailCellRendererParams -> formatDetailCellRendererParams

        Returns:
            The JavaScript formatter function name.
        """
        return f"format{cls.__name__}"



"""Map demo showcasing vector layers."""

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.map.types import LatLng, latlng, latlng_bounds

from .common import demo


class VectorLayersState(rx.State):
    """State class for the vector layers map demo."""

    # Define state variables
    zoom: float = 13
    center: LatLng = latlng(lat=51.505, lng=-0.09)


@demo(
    route="/vector-layers",
    title="Vector Layers",
    description="Demonstrates different vector layers like circles, polygons, polylines, and rectangles.",
)
def vector_layers() -> rx.Component:
    # Create coordinates for our vector layers
    center = latlng(lat=51.505, lng=-0.09)
    map_id = "vector-layers-demo"

    # Create polygon coordinates
    polygon_points = [
        latlng(lat=51.51, lng=-0.1),
        latlng(lat=51.51, lng=-0.08),
        latlng(lat=51.5, lng=-0.07),
        latlng(lat=51.49, lng=-0.08),
        latlng(lat=51.49, lng=-0.1),
    ]

    # Create polyline coordinates
    polyline_points = [
        latlng(lat=51.52, lng=-0.12),
        latlng(lat=51.53, lng=-0.11),
        latlng(lat=51.51, lng=-0.06),
        latlng(lat=51.52, lng=-0.05),
    ]

    # Create rectangle bounds
    rect_bounds = latlng_bounds(
        corner1_lat=51.49,  # Southwest corner
        corner1_lng=-0.13,
        corner2_lat=51.51,  # Northeast corner
        corner2_lng=-0.11,
    )

    # Create path options for styling
    circle_options = rxe.map.path_options(
        color="#ff0000",
        fill_color="#ff3333",
        fill_opacity=0.5,
    )

    polygon_options = rxe.map.path_options(
        color="#0033ff",
        fill_color="#3366ff",
        fill_opacity=0.4,
        weight=2,
    )

    polyline_options = rxe.map.path_options(
        color="#166516FF",
        weight=5,
        opacity=0.7,
        line_cap="round",
        dash_array="5,10",
    )

    rectangle_options = rxe.map.path_options(
        color="#ff9900",
        fill_color="#ffcc00",
        fill_opacity=0.3,
        weight=3,
    )

    return rx.vstack(
        rx.heading("Vector Layers Demo"),
        rx.text("This map demonstrates different vector layers available in Leaflet."),
        rx.button(
            "Get Bounds",
            on_click=rxe.map.api(map_id).get_bounds(
                callback=rx.console_log,
            ),
        ),
        rxe.map(
            # Base tile layer
            rxe.map.tile_layer(
                url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
            ),
            # Add Circle
            rxe.map.circle(
                center=center,
                radius=500,  # 500 meters
                path_options=circle_options,
            ),
            # Add CircleMarker (radius in pixels)
            rxe.map.circle_marker(
                rxe.map.tooltip("Circle Marker A"),
                center=latlng(lat=51.515, lng=-0.09),
                radius=15,  # 15 pixels
                path_options=circle_options,
            ),
            # Add Polygon
            rxe.map.polygon(
                positions=polygon_points,
                path_options=polygon_options,
            ),
            # Add Polyline
            rxe.map.polyline(
                positions=polyline_points,
                path_options=polyline_options,
            ),
            # Add Rectangle
            rxe.map.rectangle(
                bounds=rect_bounds,
                path_options=rectangle_options,
            ),
            # Add markers to help with orientation
            rxe.map.marker(
                rxe.map.tooltip("Center point"),
                position=latlng(lat=51.505, lng=-0.09),
            ),
            # Map settings
            id=map_id,
            zoom=VectorLayersState.zoom,
            center=VectorLayersState.center,
            width="100%",
            height="50vh",
        ),
        rx.text(
            "The map shows: a red circle (500m radius), a red circle marker (15px), a blue polygon, "
            "a green dashed polyline, and an orange rectangle."
        ),
        width="100%",
        padding="1em",
    )



from typing import Any, Mapping, TypedDict

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.flow.types import Edge, Node, Position

initial_edges: list[Edge] = [
    {
        "id": "e1-2",
        "source": "1-1",
        "target": "1-2",
        "label": "edge",
        "type": "smoothstep",
    },
    {
        "id": "e1-3",
        "source": "1-1",
        "target": "1-3",
        "animated": True,
        "label": "animated edge",
    },
    {
        "id": "e2-2",
        "source": "1-2",
        "target": "2-2",
        "type": "smoothstep",
        "markerEnd": {
            "type": "arrowclosed",
        },
    },
    {
        "id": "e2-3",
        "source": "2-2",
        "target": "2-3",
        "type": "smoothstep",
        "markerEnd": {
            "type": "arrowclosed",
        },
    },
    {
        "id": "e3-3",
        "source": "2-3",
        "sourceHandle": "a",
        "target": "3-2",
        "type": "button",
        "animated": True,
        "style": {"stroke": "rgb(158, 118, 255)"},
    },
    {
        "id": "e3-4",
        "source": "2-3",
        "sourceHandle": "b",
        "target": "3-1",
        "type": "button",
    },
]

initial_nodes: list[Node] = [
    {
        "id": "annotation-1",
        "type": "annotation",
        "draggable": False,
        "selectable": False,
        "data": {
            "level": 1,
            "label": "Built-in node and edge types. Draggable, deletable and connectable!",
            "arrowStyle": {
                "right": 0,
                "bottom": 0,
                "transform": "translate(-30px,10px) rotate(-80deg)",
            },
        },
        "position": {"x": -200, "y": -30},
    },
    {
        "id": "1-1",
        "type": "input",
        "data": {
            "label": "Input Node",
        },
        "position": {"x": 150, "y": 0},
    },
    {
        "id": "1-2",
        "type": "default",
        "data": {
            "label": "Default Node",
        },
        "position": {"x": 0, "y": 100},
    },
    {
        "id": "1-3",
        "type": "output",
        "data": {
            "label": "Output Node",
        },
        "position": {"x": 300, "y": 100},
    },
    {
        "id": "annotation-2",
        "type": "annotation",
        "draggable": False,
        "selectable": False,
        "data": {
            "level": 2,
            "label": "Sub flows, toolbars and resizable nodes!",
            "arrowStyle": {
                "left": 0,
                "bottom": 0,
                "transform": "translate(5px, 25px) scale(1, -1) rotate(100deg)",
            },
        },
        "position": {"x": 220, "y": 200},
    },
    {
        "id": "2-1",
        "type": "group",
        "position": {
            "x": -170,
            "y": 250,
        },
        "style": {
            "width": 380,
            "height": 180,
        },
    },
    {
        "id": "2-2",
        "data": {},
        "type": "tools",
        "position": {"x": 50, "y": 50},
        "style": {
            "width": 80,
            "height": 80,
        },
        "parentId": "2-1",
        "extent": "parent",
    },
    {
        "id": "2-3",
        "type": "resizer",
        "data": {
            "label": "Resize Me",
        },
        "position": {"x": 250, "y": 50},
        "style": {
            "width": 80,
            "height": 80,
        },
        "parentId": "2-1",
        "extent": "parent",
    },
    {
        "id": "annotation-3",
        "type": "annotation",
        "draggable": False,
        "selectable": False,
        "data": {
            "level": 3,
            "label": "Nodes and edges can be anything and are fully customizable!",
            "arrowStyle": {
                "right": 0,
                "bottom": 0,
                "transform": "translate(-35px, 20px) rotate(-80deg)",
            },
        },
        "position": {"x": -40, "y": 570},
    },
    {
        "id": "3-2",
        "type": "textinput",
        "position": {"x": 150, "y": 650},
        "data": {},
    },
    {
        "id": "3-1",
        "type": "circle",
        "position": {"x": 350, "y": 500},
        "data": {},
    },
]


@rx.memo
def button_edge(
    id: rx.Var[str],
    sourceX: rx.Var[float],  # noqa: N803
    sourceY: rx.Var[float],  # noqa: N803
    targetX: rx.Var[float],  # noqa: N803
    targetY: rx.Var[float],  # noqa: N803
    sourcePosition: rx.Var[Position],  # noqa: N803
    targetPosition: rx.Var[Position],  # noqa: N803
    markerEnd: rx.Var[str],  # noqa: N803
):
    bezier_path = rxe.components.flow.util.get_bezier_path(
        source_x=sourceX,
        source_y=sourceY,
        target_x=targetX,
        target_y=targetY,
        source_position=sourcePosition,
        target_position=targetPosition,
    )
    return rx.fragment(
        rxe.flow.base_edge(path=bezier_path.path, markerEnd=markerEnd),
        rxe.flow.edge_label_renderer(
            rx.el.div(
                rx.el.button(
                    "×",  # noqa: RUF001
                    class_name="button-edge__button",
                    on_click=rx.run_script(
                        rxe.flow.api.set_edges(
                            rx.vars.FunctionStringVar.create(
                                "Array.prototype.filter.call"
                            ).call(
                                rxe.flow.api.get_edges(),
                                rx.Var(f"((edge) => edge.id !== {id})"),
                            ),
                        )
                    ),
                ),
                class_name="button-edge__label nodrag nopan",
                transform=f"translate(-50%, -50%) translate({bezier_path.label_x}px,{bezier_path.label_y}px)",
            )
        ),
    )


emojis = ["🚀", "🔥", "✨"]


@rx.memo
def toolbar_node(data: rx.vars.ObjectVar[Mapping[str, Any]]):
    return rx.fragment(
        rxe.flow.node_toolbar(
            *[
                rx.el.button(
                    emoji,
                    key=emoji,
                    on_click=OverviewState.set_emoji(emoji),
                    custom_attrs={"aria-label": f"Select emoji {emoji}"},
                )
                for emoji in emojis
            ],
            is_visible=True,
        ),
        rx.el.div(rx.el.div(OverviewState.emoji)),
        rxe.flow.handle(
            type="source",
            position="left",
        ),
        rxe.flow.handle(
            type="target",
            position="right",
        ),
        rx.el.div(data["label"]),
    )


@rx.memo
def circle_node(
    positionAbsoluteX: rx.vars.Var[float],  # noqa: N803
    positionAbsoluteY: rx.vars.Var[float],  # noqa: N803
):
    label = f"Position x:{round(positionAbsoluteX.guess_type())} y:{round(positionAbsoluteY.guess_type())}"
    return rx.el.div(
        rx.el.div(label),
        rxe.flow.handle(
            type="target",
            position="left",
            class_name="custom-handle",
        ),
    )


@rx.memo
def resizer_node(data: rx.vars.ObjectVar[Mapping[str, Any]]):
    return rx.fragment(
        rxe.flow.node_resizer(
            min_width=1,
            min_height=1,
        ),
        rxe.flow.handle(
            type="target",
            position="left",
            class_name="custom-handle",
        ),
        rx.el.div(data["label"]),
        rx.el.div(
            rxe.flow.handle(
                id="a",
                type="source",
                position="bottom",
                class_name="resizer-node__handle custom-handle",
            ),
            rxe.flow.handle(
                id="b",
                type="source",
                position="bottom",
                class_name="resizer-node__handle custom-handle",
            ),
            class_name="resizer-node__handles",
        ),
    )


@rx.memo
def annotation_node(data: rx.vars.ObjectVar[Mapping[str, Any]]):
    return rx.fragment(
        rx.el.div(
            rx.el.div(data["level"], ".", class_name="annotation-level"),
            rx.el.div(data["label"]),
            class_name="annotation-content",
        ),
        rx.cond(
            data["arrowStyle"],
            rx.el.div(
                "⤹",
                class_name="annotation-arrow",
                style=data["arrowStyle"],
            ),
        ),
    )


dimension_attrs = ["width", "height"]


def dimension_input(attr: str):
    return rx.fragment(
        rx.el.label(f"Node {attr}"),
        rx.el.input(
            type="number",
            value=rx.cond(OverviewState.dimensions, OverviewState.dimensions[attr], 0),
            on_change=lambda value: OverviewState.set_dimensions(attr, value),
            class_name="text-input-node__input xy-theme__input nodrag",
        ),
        key=attr,
    )


@rx.memo
def dimension_node(id: rx.vars.Var[str]):
    return rx.fragment(
        *[dimension_input(attr) for attr in dimension_attrs],
        rxe.flow.handle(
            type="target",
            position="top",
            class_name="custom-handle",
        ),
    )


class Dimensions(TypedDict):
    width: float
    height: float


class OverviewState(rx.State):
    emoji: str = "🚀"

    nodes: rx.Field[list[Node]] = rx.field(default_factory=lambda: initial_nodes)
    edges: rx.Field[list[Edge]] = rx.field(default_factory=lambda: initial_edges)

    @rx.var
    def dimensions(self) -> Dimensions | None:
        node = next((node for node in self.nodes if node["id"] == "2-3"), None)
        if (
            not node
            or not (measured := node.get("measured"))
            or not (height := measured.get("height"))
            or not (width := measured.get("width"))
        ):
            return None
        return {"width": width, "height": height}

    @rx.event
    def set_emoji(self, emoji: str):
        self.emoji = emoji

    @rx.event
    def set_nodes(self, nodes: list[Node]):
        self.nodes = nodes

    @rx.event
    def set_edges(self, edges: list[Edge]):
        self.edges = edges

    @rx.event
    def set_dimensions(self, attr: str, value_str: str):
        value = float(value_str)

        def _updated_node(current_node: Node) -> Node:
            parent_node = next(
                (node for node in self.nodes if node["id"] == "2-1"), None
            )
            parent_width = (
                parent_node.get("style", {}).get("width") if parent_node else None
            ) or float("inf")
            parent_height = (
                parent_node.get("style", {}).get("height") if parent_node else None
            ) or float("inf")

            current_pos_x = current_node.get("position", {}).get("x", 0)
            current_pos_y = current_node.get("position", {}).get("y", 0)

            max_width = max(parent_width - current_pos_x, 0)
            max_height = max(parent_height - current_pos_y, 0)

            new_size = {
                "width": (
                    min(value, max_width)
                    if attr == "width"
                    else current_node.get("style", {}).get("width")
                ),
                "height": (
                    min(value, max_height)
                    if attr == "height"
                    else current_node.get("style", {}).get("height")
                ),
            }

            return {
                **current_node,
                "style": {**current_node.get("style", {}), **new_size},
            }

        self.nodes = [
            node if node["id"] != "2-3" else _updated_node(node) for node in self.nodes
        ]


@rx.page(route="/overview", title="Feature Overview")
def overview():
    return rx.box(
        rxe.flow(
            rxe.flow.mini_map(
                zoomable=True,
                pannable=True,
                node_class_name=rx.Var("((node) => node.type)"),
            ),
            rxe.flow.controls(),
            rxe.flow.background(),
            on_nodes_change=lambda node_changes: OverviewState.set_nodes(
                rxe.flow.util.apply_node_changes(OverviewState.nodes, node_changes)
            ),
            on_edges_change=lambda edge_changes: OverviewState.set_edges(
                rxe.flow.util.apply_edge_changes(OverviewState.edges, edge_changes)
            ),
            nodes=OverviewState.nodes,
            edges=OverviewState.edges,
            node_types={
                "tools": toolbar_node,
                "circle": circle_node,
                "resizer": resizer_node,
                "annotation": annotation_node,
                "textinput": dimension_node,
            },
            edge_types={"button": button_edge},
            color_mode="light",
            fit_view=True,
            attribution_position="top-right",
        ),
        height="100vh",
        width="100vw",
    )



"""Common components used by all map demo pages."""

import dataclasses
import inspect
from functools import wraps
from pathlib import Path
from typing import Callable

import reflex as rx


@dataclasses.dataclass(frozen=True)
class DemoPage:
    """A demo page."""

    route: str
    title: str
    description: str


_DEMO_PAGES: dict[str, DemoPage] = {}


class DemoState(rx.State):
    """State for the demo pages."""

    @rx.var
    def pages(self) -> list[DemoPage]:
        """List of demo pages."""
        return [p for p in _DEMO_PAGES.values() if p.route != "/"]


def demo_dropdown():
    """Dropdown to navigate between demo pages."""
    return rx.select.root(
        rx.select.trigger(placeholder="Select Demo"),
        rx.select.content(
            rx.foreach(
                DemoState.pages,
                lambda page: rx.select.item(
                    rx.heading(page.title, size="3"), value=page.route
                ),
            )
        ),
        value=rx.cond(
            rx.State.router.page.path == "/",
            "",
            rx.State.router.page.path,
        ),
        on_change=rx.redirect,
    )


def demo_template(page: Callable):
    """Template for all demo pages."""
    page_data = _DEMO_PAGES[page.__name__]
    page_file = Path(inspect.getfile(page))
    page_source = page_file.read_text()
    relative_page_file = page_file.relative_to(
        Path(__file__, "..", "..", "..", "..").resolve()
    )

    return rx.container(
        rx.hstack(
            demo_dropdown(),
            rx.spacer(),
            rx.link(
                rx.heading("Map Demos", size="4"),
                href="/",
            ),
            rx.color_mode.button(),
            width="100%",
            align="center",
        ),
        rx.text(page_data.description, margin_left="10px", margin_top="5px"),
        rx.tabs.root(
            rx.tabs.list(
                rx.tabs.trigger(
                    rx.hstack(rx.icon("eye"), rx.text("Example")), value="example"
                ),
                rx.tabs.trigger(
                    rx.hstack(rx.icon("code"), rx.text("Source")), value="source"
                ),
                margin_bottom="1em",
            ),
            rx.tabs.content(page(), value="example", height="fit-content"),
            rx.tabs.content(
                rx.card(
                    rx.inset(
                        rx.badge(
                            rx.code(str(relative_page_file)),
                            width="100%",
                            size="2",
                            height="3em",
                            radius="none",
                        ),
                        side="top",
                        pb="current",
                    ),
                    rx.code_block(
                        page_source, language="python", show_line_numbers=True
                    ),
                ),
                value="source",
            ),
            default_value="example",
            margin_top="1em",
        ),
        size="4",
    )


def demo(route: str, title: str, description: str, **kwargs):
    """Decorator to add the demo page to the demo registry."""

    def decorator(page: Callable):
        _DEMO_PAGES[page.__name__] = DemoPage(
            route=route, title=title, description=description
        )

        @rx.page(route=route, title=title, description=description, **kwargs)
        @wraps(page)
        def inner():
            return demo_template(page)

        return inner

    return decorator



"""Utility functions for Reflex Enterprise."""

import dataclasses
import inspect
from collections.abc import Sequence
from typing import Any, Callable, TypedDict

import psutil
import reflex as rx
from packaging import version
from reflex import ImportVar, constants
from reflex.config import get_config
from reflex.event import Event, IndividualEventType, fix_events
from reflex.state import BaseState, StateUpdate
from reflex.utils import console, prerequisites
from reflex.utils.decorator import once
from reflex.utils.prerequisites import get_app
from reflex.vars.base import Var, VarData
from reflex.vars.function import ArgsFunctionOperation, FunctionStringVar

from reflex_enterprise.constants import IS_OFFLINE

from .vars import ArgsFunctionOperationPromise, PromiseVar


def get_user_tier() -> str:
    """Get the current user's subscription tier.

    Returns:
        The user's subscription tier as a string.
    """
    if IS_OFFLINE:
        return "enterprise"
    return prerequisites.get_user_tier() or "anonymous"


def check_config_option_in_tier(
    option_name: str,
    allowed_tiers: list[str],
    fallback_value: Any,
    help_link: str | None = None,
):
    """Check if a config option is allowed for the authenticated user's current tier.

    Args:
        option_name: The name of the option to check.
        allowed_tiers: The tiers that are allowed to use the option.
        fallback_value: The fallback value if the option is not allowed.
        help_link: The help link to show to a user that is authenticated.
    """
    config = get_config()
    current_tier = get_user_tier()

    if allowed_tiers == []:
        the_remedy = "This option is not available in the current context."
    else:
        if current_tier == "anonymous":
            the_remedy = "You are currently logged out. Run `reflex login` to access this option."
        else:
            the_remedy = (
                f"Your current subscription tier is `{current_tier}`. "
                f"Please upgrade to {allowed_tiers} to access this option. "
            )
            if help_link:
                the_remedy += f"See {help_link} for more information."

    value = getattr(config, option_name)

    if value is None:
        setattr(config, option_name, fallback_value)
        return

    if value != fallback_value and current_tier not in allowed_tiers:
        console.warn(f"Config option `{option_name}` is restricted. {the_remedy}")
        setattr(config, option_name, fallback_value)
        config._set_persistent(**{option_name: fallback_value})


def is_deploy_context():
    """Check if the current context is a deploy context.

    Returns:
        True if the current context is a deploy context, False otherwise.
    """
    from reflex.utils.exec import get_compile_context

    return get_compile_context() == constants.CompileContext.DEPLOY  # pyright: ignore [reportPrivateImportUsage]


def get_backend_url(relative_url: str | Var[str]) -> Var[str]:
    """Get the full backend URL for a given relative URL.

    Use with `fetch` to access backend API endpoints.

    Args:
        relative_url: The relative URL to convert.

    Returns:
        A Var representing the full backend URL.
    """
    # Remove when reflex-enterprise drops support for < 0.8.27
    use_compatible_get_backend_url = version.parse(
        rx.constants.Reflex.VERSION
    ) < version.parse("0.8.27")
    if use_compatible_get_backend_url:
        # This is the older, less ideal implementation
        return ArgsFunctionOperation.create(
            args_names=("url",),
            return_expr=Var(
                r"`${getBackendURL(typeof env !== 'undefined' ? env.UPLOAD : env_default.UPLOAD).origin}/${url.replace(/^\/+/, '')}`"
            ),
            _var_data=VarData(
                imports={
                    "$/utils/state": ["getBackendURL"],
                    "$/env.json": ImportVar(tag="env", is_default=True),
                }
            ),
        )(relative_url).to(str)

    # Reflex 0.8.27+ use the newer getBackendURL that doesn't require an argument.
    return ArgsFunctionOperation.create(
        args_names=("url",),
        return_expr=rx.Var(r"`${getBackendURL().origin}/${url.replace(/^\/+/, '')}`"),
        _var_data=VarData(
            imports={
                "$/utils/state": ["getBackendURL"],
            }
        ),
    )(relative_url).to(str)


def fetch(
    url: str | Var[str], options: dict[str, Any] | Var[dict[str, Any]] | None = None
) -> PromiseVar:
    """Fetch a URL with the given options.

    Args:
        url: The URL to fetch.
        options: The options to use for the fetch.

    Returns:
        A PromiseVar representing the eventual result of the fetch.
    """
    return ArgsFunctionOperationPromise.create(
        args_names=("url", "options"),
        return_expr=Var(
            "fetch(url, {...options, headers: {...options.headers, 'X-Reflex-Client-Token': getToken()}})"
        ),
        _var_data=VarData(
            imports={
                "$/utils/state": [
                    "getToken",
                ],
            }
        ),
    )(url, options or {})


def encode_uri_component(value: str | Var[str]) -> Var[str]:
    """Encode a URI component.

    Args:
        value: The value to encode.

    Returns:
        A Var representing the encoded URI component.
    """
    return FunctionStringVar.create("encodeURIComponent")(value).to(str)


def arrow_func(py_fn: Callable) -> ArgsFunctionOperation:
    """Convert a Python function to a js arrow function."""
    sig = inspect.signature(py_fn)
    params = [
        Var(
            param.name,
            _var_type=param.annotation,
        ).guess_type()
        for param in sig.parameters.values()
    ]
    return ArgsFunctionOperation.create(
        args_names=tuple(sig.parameters),
        return_expr=py_fn(*params),
    )


@once
def is_new_session() -> bool:
    """Check if this is a new development session vs hot reload restart.

    Returns:
        True if this is a new session, False if it's a hot reload restart.
    """
    # Get current parent PID (this could fail in unusual environments)
    try:
        current_parent_pid = (
            parent.pid if (parent := psutil.Process().parent()) else None
        )
    except Exception:
        return True  # Default to showing message if process detection fails

    if current_parent_pid is None:
        return True

    # Check stored parent PID from previous session
    backend_dir = prerequisites.get_web_dir() / "backend"
    backend_dir.mkdir(parents=True, exist_ok=True)
    session_file = backend_dir / "session_parent_pid"

    if session_file.exists():
        # Read and parse stored PID (this could fail if file is corrupted)
        try:
            stored_parent_pid = int(session_file.read_text().strip())
            is_new = current_parent_pid != stored_parent_pid
        except Exception:
            # File is corrupted, treat as new session
            is_new = True
    else:
        # First time, treat as new session
        is_new = True

    if is_new:
        session_file.write_text(str(current_parent_pid))

    return is_new


class CallEventPayload(TypedDict):
    """Payload for calling an event from a computed var."""

    handler: rx.EventHandler
    state: BaseState
    payload: Any


def _call_event_payload(state: BaseState, fixed_event: Event) -> CallEventPayload:
    substate, handler = state._get_root_state()._get_event_handler(fixed_event)

    if handler.is_background or not isinstance(substate, BaseState):
        msg = "Background events cannot be called from computed vars. Use chain_event_out_of_band instead."
        raise RuntimeError(msg)
    return CallEventPayload(
        handler=handler,
        state=substate,
        payload=fixed_event.payload,
    )


async def call_event_from_computed_var(
    state: BaseState, events: IndividualEventType | list[IndividualEventType]
):
    """Chain an event from an async computed var.

    Args:
        state: The state to call the event on.
        events: The events to call.
    """
    if not isinstance(events, Sequence):
        events = [events]
    token = state.router.session.client_token

    event_payloads = [
        _call_event_payload(state, fixed_event)
        for fixed_event in fix_events(
            events=events,  # pyright: ignore[reportArgumentType]
            token=token,
            router_data=state.router_data,
        )
    ]

    app = get_app().app

    # Clean the current dirty vars, since the original ones are saved in the `get_delta` frame.
    state._clean()

    for ep in event_payloads:
        async for update in state._process_event(**ep):
            # In-bound updates are not final! The delta that triggered the computed
            # var may be final and we don't want to override that.
            update = dataclasses.replace(update, final=None)

            # Send the update to the client.
            await app.event_namespace.emit_update(update=update, token=token)


async def chain_event_out_of_band(
    state: BaseState, events: IndividualEventType | list[IndividualEventType]
):
    """Chain an event out-of-band from the main event loop.

    This allows an event handler to deterministically chain another event
    regardless of whether it is returned/yielded. It is primarily useful for
    allowing helper functions which return data to also trigger events as side
    effects.

    Args:
        state: The state associated with the client to call the event on.
        events: The event to chain.
    """
    if not isinstance(events, Sequence):
        events = [events]
    token = state.router.session.client_token
    app = get_app().app
    # Send the update to the client.
    await app.event_namespace.emit_update(
        update=StateUpdate(
            events=fix_events(
                events=events,  # pyright: ignore[reportArgumentType]
                token=token,
                router_data=state.router_data,
            ),
            final=None,
        ),
        token=token,
    )



"""Tree example using ag-grid."""

import json
from typing import Any

import reflex as rx
from reflex.vars.base import Var

import reflex_enterprise as rxe

from .common import demo

human_size = rx.vars.function.ArgsFunctionOperation.create(
    ["params"],
    Var("""{const sizeInKb = params.value / 1024;

    if (sizeInKb > 1024) {
        return `${+(sizeInKb / 1024).toFixed(2)} MB`;
    } else {
        return `${+sizeInKb.toFixed(2)} KB`;
    }}"""),
)


class TreeDisplayState(rx.State):
    """State for the tree display."""

    data: list[dict[str, Any]] = [
        {
            "host": "vali",
            "path": ["Desktop", "ProjectAlpha", "Proposal.docx"],
            "size": 512000,
            "created": "2023-07-10",
            "modified": "2023-08-01",
        },
        {
            "host": "vali",
            "path": ["Desktop", "ProjectAlpha", "Timeline.xlsx"],
            "size": 1048576,
            "created": "2023-07-12",
            "modified": "2023-08-03",
        },
        {
            "host": "vali",
            "path": ["Desktop", "ToDoList.txt"],
            "size": 51200,
            "created": "2023-08-05",
            "modified": "2023-08-10",
        },
        {
            "host": "vali",
            "path": ["Desktop", "MeetingNotes_August.pdf"],
            "size": 460800,
            "created": "2023-08-15",
            "modified": "2023-08-15",
        },
        {
            "host": "vidar",
            "path": ["Desktop", "LaunchCodes.txt"],
            "size": 32500,
            "created": "1973-08-05",
            "modified": "2023-08-10",
        },
        {
            "host": "vidar",
            "path": ["Desktop", "funtime.pdf"],
            "size": 460800,
            "created": "2023-08-15",
            "modified": "2023-08-15",
        },
        {
            "host": "vali",
            "path": ["Documents", "Work", "ProjectAlpha", "Proposal.docx"],
            "size": 512000,
            "created": "2023-07-10",
            "modified": "2023-08-01",
        },
        {
            "host": "vali",
            "path": ["Documents", "Work", "ProjectAlpha", "Timeline.xlsx"],
            "size": 1048576,
            "created": "2023-07-12",
            "modified": "2023-08-03",
        },
        {
            "host": "vali",
            "path": ["Documents", "Work", "ProjectBeta", "Report.pdf"],
            "size": 1024000,
            "created": "2023-06-22",
            "modified": "2023-07-15",
        },
        {
            "host": "vali",
            "path": ["Documents", "Work", "ProjectBeta", "Budget.xlsx"],
            "size": 1048576,
            "created": "2023-06-25",
            "modified": "2023-07-18",
        },
        {
            "host": "vidar",
            "path": ["Documents", "Work", "Meetings", "TeamMeeting_August.pdf"],
            "size": 512000,
            "created": "2023-08-20",
            "modified": "2023-08-21",
        },
        {
            "host": "vidar",
            "path": ["Documents", "Work", "Meetings", "ClientMeeting_July.pdf"],
            "size": 1048576,
            "created": "2023-07-15",
            "modified": "2023-07-16",
        },
        {
            "host": "vali",
            "path": ["Documents", "Personal", "Taxes", "2022.pdf"],
            "size": 1024000,
            "created": "2023-04-10",
            "modified": "2023-04-10",
        },
        {
            "host": "vali",
            "path": ["Documents", "Personal", "Taxes", "2021.pdf"],
            "size": 1048576,
            "created": "2022-04-05",
            "modified": "2022-04-06",
        },
        {
            "host": "vali",
            "path": ["Documents", "Personal", "Taxes", "2020.pdf"],
            "size": 1024000,
            "created": "2021-04-03",
            "modified": "2021-04-03",
        },
        {
            "host": "vali",
            "path": ["Pictures", "Vacation2019", "Beach.jpg"],
            "size": 1048576,
            "created": "2019-07-10",
            "modified": "2019-07-12",
        },
        {
            "host": "vali",
            "path": ["Pictures", "Vacation2019", "Mountain.png"],
            "size": 2048000,
            "created": "2019-07-11",
            "modified": "2019-07-13",
        },
        {
            "host": "vali",
            "path": ["Pictures", "Family", "Birthday2022.jpg"],
            "size": 3072000,
            "created": "2022-12-15",
            "modified": "2022-12-20",
        },
        {
            "host": "vali",
            "path": ["Pictures", "Family", "Christmas2021.png"],
            "size": 2048000,
            "created": "2021-12-25",
            "modified": "2021-12-26",
        },
        {
            "host": "vali",
            "path": ["Videos", "Vacation2019", "Beach.mov"],
            "size": 4194304,
            "created": "2019-07-10",
            "modified": "2019-07-12",
        },
        {
            "host": "vali",
            "path": ["Videos", "Vacation2019", "Hiking.mp4"],
            "size": 4194304,
            "created": "2019-07-15",
            "modified": "2019-07-16",
        },
        {
            "host": "vali",
            "path": ["Videos", "Family", "Birthday2022.mp4"],
            "size": 6291456,
            "created": "2022-12-15",
            "modified": "2022-12-20",
        },
        {
            "host": "vali",
            "path": ["Videos", "Family", "Christmas2021.mov"],
            "size": 6291456,
            "created": "2021-12-25",
            "modified": "2021-12-26",
        },
        {
            "host": "vidar",
            "path": ["Downloads", "SoftwareInstaller.exe"],
            "size": 2097152,
            "created": "2023-08-01",
            "modified": "2023-08-01",
        },
        {
            "host": "vidar",
            "path": ["Downloads", "Receipt_OnlineStore.pdf"],
            "size": 1048576,
            "created": "2023-08-05",
            "modified": "2023-08-05",
        },
        {
            "host": "vali",
            "path": ["Downloads", "Ebook.pdf"],
            "size": 1048576,
            "created": "2023-08-08",
            "modified": "2023-08-08",
        },
    ]
    combine_hosts: rx.Field[bool] = rx.field(True)


class GridState(rx.State):
    """State for the grid demo."""

    column_state_json: str = rx.LocalStorage()

    @rx.event
    def save_column_state(self, state: list):
        """Event handler to save the column state."""
        self.column_state_json = json.dumps(state)

    @rx.var
    def column_state(self) -> list:
        """Get the column state from the json."""
        try:
            return json.loads(self.column_state_json)
        except ValueError:
            return []


@demo(
    route="/tree",
    title="Tree (enterprise)",
    description="Use tree data with get_data_path to visualize hierarchical information.",
)
def tree_example():
    """Tree example."""
    return rx.box(
        rx.hstack(
            "Combine Hosts",
            rx.switch(
                checked=TreeDisplayState.combine_hosts,
                on_change=TreeDisplayState.setvar("combine_hosts"),
            ),
            rx.button(
                "Save column state",
                on_click=rxe.ag_grid.api("ag_grid_tree_1").get_column_state(  # pyright: ignore [reportAttributeAccessIssue]
                    callback=GridState.save_column_state
                ),
            ),
            rx.button(
                "Load column state",
                on_click=rxe.ag_grid.api("ag_grid_tree_1").apply_column_state(  # pyright: ignore [reportAttributeAccessIssue]
                    {"state": GridState.column_state, "applyOrder": True}
                ),
                disabled=GridState.column_state.length() == 0,
            ),
            align="center",
        ),
        rxe.ag_grid.root(
            id="ag_grid_tree_1",
            row_data=TreeDisplayState.data,
            auto_group_column_def=rxe.ag_grid.column_def(  # pyright: ignore [reportCallIssue]
                field="",
                header_name="File Explorer",
                min_width=280,
                cell_renderer_params={"suppressCount": True},
            ),
            column_defs=[  # pyright: ignore [reportArgumentType]
                rxe.ag_grid.column_def(field="created"),  # pyright: ignore [reportCallIssue]
                rxe.ag_grid.column_def(field="modified"),  # pyright: ignore [reportCallIssue]
                rxe.ag_grid.column_def(  # pyright: ignore [reportCallIssue]
                    field="size",
                    agg_func="sum",
                    value_formatter=human_size,
                ),
                rxe.ag_grid.column_def(  # pyright: ignore [reportCallIssue]
                    field="host",
                    hide=~TreeDisplayState.combine_hosts,
                    agg_func=rx.vars.FunctionStringVar.create(
                        "(params) => params.values.join(', ').split(', ').filter((value, index, self) => self.indexOf(value) === index).join(', ')"
                    ),
                ),
            ],
            default_column_def={"flex": 1},
            group_default_expanded=Var.create(0),
            tree_data=True,
            # get_data_path is an Initial option, it cannot be set after initialization.
            get_data_path=rx.cond(
                TreeDisplayState.combine_hosts,
                rx.vars.function.ArgsFunctionOperation.create(
                    ["data"],
                    Var("data.path"),
                ),
                rx.vars.function.ArgsFunctionOperation.create(
                    ["data"],
                    Var("[data.host, ...data.path]"),
                ),
            ).to(rx.vars.FunctionVar),
            # This key causes the grid to re-initialize when `combine_hosts` var changes
            key=f"ag_grid_{TreeDisplayState.combine_hosts}",
        ),
        width="100%",
        height="71vh",
        padding_bottom="25px",  # for scroll bar
    )



"""Mantines Dates."""

from typing import Callable, Literal, TypedDict

import reflex as rx
from reflex.components.component import Component

from reflex_enterprise.components.component import ComponentEnterprise

from .base import (
    MantineDescriptionProps,
    MantineErrorProps,
    MantineLabelProps,
    MantineLeftSection,
    MantineRightSection,
    MemoizedMantineProvider,
)

MANTINE_DATES_PACKAGE = "@mantine/dates"
MANTINE_DATES_VERSION = "8.3.9"


class MantineDates(ComponentEnterprise):
    """Mantine Dates component."""

    library = f"{MANTINE_DATES_PACKAGE}@{MANTINE_DATES_VERSION}"

    lib_dependencies: list[str] = ["dayjs@1.11.19"]

    def add_imports(self):
        """Add import for Mantine Dates component."""
        return {
            "": [
                "@mantine/core/styles.css",
                "@mantine/dates/styles.css",
            ],
            "dayjs": [rx.ImportVar(tag="dayjs", is_default=True)],
        }

    @staticmethod
    def _get_app_wrap_components() -> dict[tuple[int, str], Component]:
        return {
            (44, "MantineProvider"): MemoizedMantineProvider.create(),
        }


class CalendarEvents(ComponentEnterprise):
    """Events for Calendar component."""

    on_change: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when date changes
    on_date_change: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when level changes
    on_level_change: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when user selects month
    on_month_select: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when the next decade button is clicked
    on_next_decade: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when the next month button is clicked
    on_next_month: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when the next year button is clicked
    on_next_year: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when the previous decade button is clicked
    on_previous_decade: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when the previous month button is clicked
    on_previous_month: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when the previous year button is clicked
    on_previous_year: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when mouse enters year control
    on_year_control_mouse_enter: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when user selects year
    on_year_select: rx.EventHandler[rx.event.passthrough_event_spec(str)]


class PresetEntry(TypedDict):
    """Preset entry for date picker."""

    value: str | tuple[str, str]
    label: str


class SharedCalendarProps(ComponentEnterprise):
    """Shared props for Calendar and DatePicker components."""

    # aria-label attributes for controls on different levels
    aria_labels: rx.Var[dict[str, str]]

    # Number of columns to scroll with next/prev buttons, same as numberOfColumns if not set explicitly
    columns_to_scroll: rx.Var[int]

    # Displayed date in controlled mode
    date: rx.Var[str]

    # dayjs format for decade label or a function that returns decade label based on the date value, "YYYY" by default
    decade_label_format: rx.Var[str]

    # Initial displayed date in uncontrolled mode
    default_date: rx.Var[str]

    # Initial displayed level in uncontrolled mode
    default_level: rx.Var[Literal["decade", "year", "month"]]

    # Number 0-6, where 0 - Sunday and 6 - Saturday. 1 - Monday by default
    first_day_of_week: rx.Var[int]

    # Assigns aria-label to Day components based on date
    get_day_aria_label: rx.Var[Callable]

    # Passes props down to Day components
    get_day_props: rx.Var[Callable]

    # Passes props down month picker control
    get_month_control_props: rx.Var[Callable]

    # Passes props down to year picker control based on date
    get_year_control_props: rx.Var[Callable]

    # Determines whether outside dates should be hidden, false by default
    hide_outside_dates: rx.Var[bool]

    # Determines whether outside months should be hidden, false by default
    hide_weekdays: rx.Var[bool]

    # Determines whether today should be highlighted with a border, false by default
    highlight_today: rx.Var[bool]

    # Current displayed level displayed in controlled mode
    level: rx.Var[Literal["decade", "year", "month"]]

    # Dayjs locale, defaults to value defined in DatesProvider
    locale: rx.Var[str]

    # Maximum possible date in YYYY-MM-DD format or Date object
    max_date: rx.Var[str]

    # Max level that user can go up to (decade, year, month), defaults to decade
    max_level: rx.Var[Literal["decade", "year", "month"]]

    # Minimum possible date in YYYY-MM-DD format or Date object
    min_date: rx.Var[str]

    # Minimum level that user can go down to (decade, year, month), defaults to month
    min_level: rx.Var[Literal["decade", "year", "month"]]

    # dayjs label format to display month label or a function that returns month label based on month value, "MMMM YYYY"
    month_label_format: rx.Var[str]

    # dayjs format for months list
    month_format: rx.Var[str]

    # Change next icon
    next_icon: rx.Var[Component]

    # Next button aria-label
    next_label: rx.Var[str]

    # Number of columns displayed next to each other, 1 by default
    number_of_columns: rx.Var[int]

    # Presets for quick date selection
    presets: rx.Var[list[PresetEntry]]

    # Change previous icon
    previous_icon: rx.Var[Component]

    # Previous button aria-label
    previous_label: rx.Var[str]

    # Controls day value rendering
    render_day: rx.Var[Callable]

    # Component size
    size: rx.Var[Literal["xs", "sm", "md", "lg", "xl"]]

    # dayjs format for weekdays names, 'dd' by default
    weekday_format: rx.Var[str]

    # Indices of weekend days, 0-6, where 0 is Sunday and 6 is Saturday. The default value is defined by DatesProvider
    weekend_days: rx.Var[list[int]]

    # Determines whether controls should be separated, true by default
    with_cell_spacing: rx.Var[bool]

    # Determines whether week numbers should be displayed, false by default
    with_week_numbers: rx.Var[bool]

    # dayjs label format to display year label or a function that returns year label based on year value, "YYYY" by default
    year_label_format: rx.Var[str]

    # dayjs format for years list
    year_list_format: rx.Var[str]


class BareCalendar(
    MantineDates,
    SharedCalendarProps,
    CalendarEvents,
):
    """BareCalendar component."""


class Calendar(BareCalendar):
    """Calendar component."""

    tag = "Calendar"

    alias = "MantineCalendar"

    # Determines whether next level button should be enabled, true by default
    has_next_level: rx.Var[bool]

    # Called when mouse enters month control
    on_month_control_mouse_enter: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Determines whether days should be static, static days can be used to display month if it is not expected that user will interact with the component in any way
    static: rx.Var[bool]


class BareCalendarPicker(
    BareCalendar,
    MantineLabelProps,
    MantineErrorProps,
    MantineDescriptionProps,
    MantineLeftSection,
    MantineRightSection,
):
    """BareCalendarPicker component."""

    placeholder: rx.Var[str]


class DateTimePicker(BareCalendarPicker):
    """DateTimePicker component."""

    tag = "DateTimePicker"

    alias = "MantineDateTimePicker"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Determines whether user can deselect the date by clicking on selected item, applicable only when type="default"
    allow_deselect: rx.Var[bool]

    # Determines whether a single day can be selected as range, applicable only when type="range"
    allow_single_day_range: rx.Var[bool]

    # Initial displayed date in uncontrolled mode
    default_date: rx.Var[str]

    # dayjs format for decade label or a function that returns decade label based on the date value, "YYYY" by default
    decade_label_format: rx.Var[str]

    # Number 0-6, where 0 - Sunday and 6 - Saturday. 1 - Monday by default
    first_day_of_week: rx.Var[int]

    # Passes props down to Day components
    get_day_props: rx.Var[Callable]


class DatePicker(BareCalendarPicker):
    """DatePicker component."""

    tag = "DatePicker"

    alias = "MantineDatePicker"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Picker type: range, multiple or default
    type: rx.Var[Literal["range", "multiple", "default"]]

    # Determines whether a single day can be selected as range, applicable only when type="range"
    allow_single_day_range: rx.Var[bool]


class DatePickerInput(BareCalendarPicker):
    """DatePickerInput component."""

    tag = "DatePickerInput"

    alias = "MantineDatePickerInput"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Picker type: range, multiple or default
    type: rx.Var[Literal["range", "multiple", "default"]]

    # Determines whether the dropdown is closed when date is selected, not applicable with type="multiple", true by default
    close_on_change: rx.Var[bool]

    # Determines whether a single day can be selected as range, applicable only when type="range"
    allow_single_day_range: rx.Var[bool]


class DateInput(BareCalendarPicker):
    """DateInput component."""

    tag = "DateInput"

    alias = "MantineDateInput"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Determines whether the dropdown is closed when date is selected, not applicable with type="multiple", true by default
    close_on_change: rx.Var[bool]

    # Determines whether a single day can be selected as range, applicable only when type="range"
    allow_single_day_range: rx.Var[bool]


class MonthPicker(BareCalendarPicker):
    """MonthPicker component."""

    tag = "MonthPicker"

    alias = "MantineMonthPicker"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Initial displayed date in uncontrolled mode
    default_date: rx.Var[str]


class MonthPickerInput(BareCalendarPicker):
    """MonthPickerInput component."""

    tag = "MonthPickerInput"

    alias = "MantineMonthPickerInput"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Initial displayed date in uncontrolled mode
    default_date: rx.Var[str]


class YearPicker(BareCalendarPicker):
    """YearPicker component."""

    tag = "YearPicker"

    alias = "MantineYearPicker"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Initial displayed date in uncontrolled mode
    default_date: rx.Var[str]


class YearPickerInput(BareCalendarPicker):
    """YearPickerInput component."""

    tag = "YearPickerInput"

    alias = "MantineYearPickerInput"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Initial displayed date in uncontrolled mode
    default_date: rx.Var[str]


class TimeInput(BareCalendarPicker):
    """TimeInput component."""

    tag = "TimeInput"

    alias = "MantineTimeInput"

    on_change: rx.EventHandler[rx.event.input_event]


class TimePicker(BareCalendarPicker):
    """TimePicker component."""

    tag = "TimePicker"

    alias = "MantineTimePicker"

    # Displayed date in controlled mode
    date: rx.Var[str]

    # Initial displayed date in uncontrolled mode
    default_date: rx.Var[str]


class TimeGrid(BareCalendarPicker):
    """TimeGrid component."""

    tag = "TimeGrid"

    alias = "MantineTimeGrid"

    # Time data in 24h format to be displayed in the grid, for example ['10:00', '18:30', '22:00']. Time values must be unique.
    data: rx.Var[list[str]]


class TimeValue(MantineDates):
    """TimeValue component."""

    tag = "TimeValue"

    alias = "MantineTimeValue"

    # AM/PM labels, { am: 'AM', pm: 'PM' } by default
    am_pm_labels: rx.Var[dict[Literal["am", "pm"], str]]

    # Time format, '24h' by default
    format: rx.Var[Literal["12h", "24h"]]

    # Time to format
    value: rx.Var[str]

    # Determines whether seconds should be displayed, false by default
    with_seconds: rx.Var[bool]


class DatesNamespace(rx.ComponentNamespace):
    """Namespace for Mantine Dates components."""

    calendar = staticmethod(Calendar.create)
    date_time_picker = staticmethod(DateTimePicker.create)
    date_picker = staticmethod(DatePicker.create)
    date_picker_input = staticmethod(DatePickerInput.create)
    date_input = staticmethod(DateInput.create)
    month_picker = staticmethod(MonthPicker.create)
    month_picker_input = staticmethod(MonthPickerInput.create)
    year_picker = staticmethod(YearPicker.create)
    year_picker_input = staticmethod(YearPickerInput.create)
    time_input = staticmethod(TimeInput.create)
    time_picker = staticmethod(TimePicker.create)
    time_grid = staticmethod(TimeGrid.create)
    time_value = staticmethod(TimeValue.create)


dates = DatesNamespace()



"""Pill component for displaying labels with optional remove button."""

from typing import Any, Literal

import reflex as rx
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import MantineCoreBase


class Pill(MantineCoreBase):
    """Displays label with optional remove button."""

    tag = "Pill"

    # Controls component size; `'sm'` by default
    size: Var[Literal["xs", "sm", "md", "lg", "xl"]]

    # Determines whether the remove button should be displayed.
    with_remove_button: Var[bool]

    # Props passed down to the remove button. Ignored if with_remove_button is false.
    remove_button_props: Var[dict[str, Any]]

    # Key of theme.radius or any valid CSS value to set border-radius, numbers are converted to rem, 0 corresponds to 0px radius. Defaults to theme default radius for the component size.
    radius: Var[str | int]

    # Disabled state.
    disabled: Var[bool]

    # Controls appearance based on theme.color value, supports any color key of theme.colors.
    variant: Var[str]

    # Called when the remove button is clicked.
    on_remove: rx.EventHandler[lambda: []]


class PillGroup(MantineCoreBase):
    """Manages spacing between Pills."""

    tag = "Pill.Group"

    # Controls size of Pills within the group. Affects font-size and padding.
    size: Var[Literal["xs", "sm", "md", "lg", "xl"]]

    # Controls gap between pills. Defaults to `theme.spacing.xs`.
    gap: Var[str | int]

    # Disabled state.
    disabled: Var[bool]


# Namespace for Pill components
class PillNamespace(rx.ComponentNamespace):
    """Namespace for Pill components."""

    __call__ = staticmethod(Pill.create)
    group = staticmethod(PillGroup.create)


pill = PillNamespace()
